@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -45,10 +45,6 @@
 #include "PView.h"
 #endif
 
-#if defined(HAVE_MESH)
-#include "periodical.h"
-#endif
-
 #if defined(HAVE_PARSER)
 #include "Parser.h"
 #endif
@@ -81,16 +77,19 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
                                        "p3d, diff, med, ...)"));
   s.push_back(mp("-bin",               "Use binary format when available"));
   s.push_back(mp("-refine",            "Perform uniform mesh refinement, then exit"));
+  s.push_back(mp("-reclassify",        "Reclassify mesh, then exit"));
   s.push_back(mp("-part int",          "Partition after batch mesh generation"));
   s.push_back(mp("-partWeight tri|quad|tet|prism|hex int", "Weight of a triangle/quad/etc. "
                                                            "during partitioning"));
-  s.push_back(mp("-saveall",           "Save all elements (discard physical group definitions)"));
-  s.push_back(mp("-parametric",        "Save vertices with their parametric coordinates"));
+  s.push_back(mp("-save_all",          "Save all elements (discard physical group definitions)"));
+  s.push_back(mp("-save_parametric",   "Save vertices with their parametric coordinates"));
+  s.push_back(mp("-save_topology",     "Save model topology"));
   s.push_back(mp("-algo string",       "Select mesh algorithm (meshadapt, del2d, front2d, "
                                         "delquad, del3d, front3d, mmg3d, pack)"));
   s.push_back(mp("-smooth int",        "Set number of mesh smoothing steps"));
   s.push_back(mp("-order int",         "Set mesh order (1, ..., 5)"));
   s.push_back(mp("-optimize[_netgen]", "Optimize quality of tetrahedral elements"));
+  s.push_back(mp("-optimize_threshold", "Optimize tetrahedral elements that have a qulaity less than a threshold"));
   s.push_back(mp("-optimize_ho",       "Optimize high order meshes"));
   s.push_back(mp("-ho_[min,max,nlayers]", "High-order optimization parameters"));
   s.push_back(mp("-optimize_lloyd",    "Optimize 2D meshes using Lloyd algorithm"));
@@ -107,7 +106,9 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-rand float",        "Set random perturbation factor"));
   s.push_back(mp("-bgm file",          "Load background mesh from file"));
   s.push_back(mp("-check",             "Perform various consistency checks on mesh"));
-  s.push_back(mp("-ignorePartBound",   "Ignore partitions boundaries"));
+  s.push_back(mp("-ignorePartBound",   "Ignore partition boundaries")); 
+  s.push_back(mp("-ignorePeriocity",   "Ignore periodic boundaries"));
+  s.push_back(mp("-oneFilePerPart",    "Save mesh partitions in separate files"));
 #if defined(HAVE_FLTK)
   s.push_back(mp("Post-processing options:", ""));
   s.push_back(mp("-link int",          "Select link mode between views (0, 1, 2, 3, 4)"));
@@ -176,10 +177,15 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("2 or F2",        "Mesh surfaces"));
   s.push_back(mp("3 or F3",        "Mesh volumes"));
   s.push_back(mp("Escape",         "Cancel lasso zoom/selection, toggle mouse selection ON/OFF"));
+  s.push_back(mp("e",              "End/accept selection in geometry creation mode"));
   s.push_back(mp("g",              "Go to geometry module"));
   s.push_back(mp("m",              "Go to mesh module"));
   s.push_back(mp("p",              "Go to post-processing module"));
+  s.push_back(mp("q",              "Abort selection in geometry creation mode"));
   s.push_back(mp("s",              "Go to solver module"));
+  s.push_back(mp("x",              "Toogle x coordinate freeze in geometry creation mode"));
+  s.push_back(mp("y",              "Toogle y coordinate freeze in geometry creation mode"));
+  s.push_back(mp("z",              "Toogle z coordinate freeze in geometry creation mode"));
   s.push_back(mp("Shift+a",        "Bring all windows to front"));
   s.push_back(mp("Shift+g",        "Show geometry options"));
   s.push_back(mp("Shift+m",        "Show mesh options"));
@@ -188,8 +194,12 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("Shift+s",        "Show solver options"));
   s.push_back(mp("Shift+u",        "Show post-processing view plugins"));
   s.push_back(mp("Shift+w",        "Show post-processing view options"));
+  s.push_back(mp("Shift+x",        "Move only along x coordinate in geometry creation mode"));
+  s.push_back(mp("Shift+y",        "Move only along y coordinate in geometry creation mode"));
+  s.push_back(mp("Shift+z",        "Move only along z coordinate in geometry creation mode"));
   s.push_back(mp("Shift+Escape",   "Enable full mouse selection"));
   s.push_back(mp(cc + "d",         "Attach/detach menu"));
+  s.push_back(mp(cc + "e",         "Export project"));
   s.push_back(mp(cc + "f",         "Enter full screen"));
   s.push_back(mp(cc + "i",         "Show statistics window"));
   s.push_back(mp(cc + "j",         "Save model options"));
@@ -201,14 +211,13 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp(cc + "o",         "Open project file"));
   s.push_back(mp(cc + "q",         "Quit"));
   s.push_back(mp(cc + "r",         "Rename project file"));
-  s.push_back(mp(cc + "s",         "Save file as"));
+  s.push_back(mp(cc + "s",         "Save mesh in default format"));
   s.push_back(mp("Shift+" + cc + "c", "Show clipping plane window"));
   s.push_back(mp("Shift+" + cc + "h", "Show current options and workspace window"));
   s.push_back(mp("Shift+" + cc + "j", "Save options as default"));
   s.push_back(mp("Shift+" + cc + "m", "Show manipulator window"));
   s.push_back(mp("Shift+" + cc + "n", "Show option window"));
   s.push_back(mp("Shift+" + cc + "o", "Merge file(s)"));
-  s.push_back(mp("Shift+" + cc + "s", "Save mesh in default format"));
   s.push_back(mp("Shift+" + cc + "u", "Show plugin window"));
   s.push_back(mp("Shift+" + cc + "v", "Show visibility window"));
   s.push_back(mp("Alt+a",          "Loop through axes modes"));
@@ -376,6 +385,10 @@ void GetOptions(int argc, char *argv[])
         CTX::instance()->batch = 5;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "reclassify")) {
+        CTX::instance()->batch = 6;
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "renumber")) {
         CTX::instance()->batchAfterMesh = 1;
         CTX::instance()->partitionOptions.renumber = 1;
@@ -480,9 +493,21 @@ void GetOptions(int argc, char *argv[])
         i++;
       }
       else if(!strcmp(argv[i] + 1, "optimize")) {
+	Msg::Warning("The '-optimize' option is now obsolete: "
+                     "Gmsh optimizes tetrahedral meshes by default");
+	Msg::Warning("Use '-optimize_threshold threshold' to "
+                     "control which elements are optimized");
+	Msg::Warning("Option '-optimize_threshold 0' leads to no optimization");
         CTX::instance()->mesh.optimize = 1;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "optimize_threshold")) {
+        i++;
+        if(argv[i])
+          opt_mesh_optimize_threshold(0, GMSH_SET, atof(argv[i++]));
+        else
+          Msg::Fatal("Missing number");
+      }
       else if(!strcmp(argv[i] + 1, "optimize_netgen")) {
         CTX::instance()->mesh.optimizeNetgen = 1;
         i++;
@@ -520,16 +545,6 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing number of lloyd iterations");
       }
-#if defined(HAVE_MESH)
-      else if(!strcmp(argv[i] + 1, "microstructure")) {
-        i++;
-        if(argv[i]) microstructure(argv[i++]);
-      }
-      else if(!strcmp(argv[i] + 1, "computeBestSeeds")) {
-        i++;
-        if(argv[i]) computeBestSeeds(argv[i++]);
-      }
-#endif
       else if(!strcmp(argv[i] + 1, "nopopup")) {
         CTX::instance()->noPopup = 1;
         i++;
@@ -630,14 +645,14 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing file name");
       }
-      else if(!strcmp(argv[i] + 1, "nw")) {
+      else if(!strcmp(argv[i] + 1, "windows")) {
         i++;
         if(argv[i])
           CTX::instance()->numWindows = atoi(argv[i++]);
         else
           Msg::Fatal("Missing number");
       }
-      else if(!strcmp(argv[i] + 1, "nt")) {
+      else if(!strcmp(argv[i] + 1, "tiles")) {
         i++;
         if(argv[i])
           CTX::instance()->numTiles = atoi(argv[i++]);
@@ -742,6 +757,14 @@ void GetOptions(int argc, char *argv[])
         i++;
         opt_mesh_ignore_part_bound(0, GMSH_SET, 1);
       }
+      else if(!strcmp(argv[i] + 1, "oneFilePerPart")) {
+        i++;
+        opt_mesh_msh_file_partitioned(0, GMSH_SET, 1);
+      }
+      else if(!strcmp(argv[i] + 1, "ignorePeriodicity")) {
+        i++;
+        opt_mesh_ignore_periodicity(0, GMSH_SET, 1);
+      }
       else if(!strcmp(argv[i] + 1, "edgelmin")) {
         i++;
         if(argv[i]) {
@@ -822,10 +845,16 @@ void GetOptions(int argc, char *argv[])
         i++;
         CTX::instance()->mesh.binary = 1;
       }
-      else if(!strcmp(argv[i] + 1, "parametric")) {
+      else if(!strcmp(argv[i] + 1, "save_parametric") ||
+              !strcmp(argv[i] + 1, "parametric")) {
         i++;
         CTX::instance()->mesh.saveParametric = 1;
       }
+      else if(!strcmp(argv[i] + 1, "save_topology") ||
+              !strcmp(argv[i] + 1, "save_entities")) {
+        i++;
+        CTX::instance()->mesh.saveTopology = 1;
+      }
       else if(!strcmp(argv[i] + 1, "algo")) {
         i++;
         if(argv[i]) {
@@ -985,7 +1014,7 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "help") || !strcmp(argv[i] + 1, "-help")) {
         fprintf(stderr, "Gmsh, a 3D mesh generator with pre- and post-processing facilities\n");
-        fprintf(stderr, "Copyright (C) 1997-2016 Christophe Geuzaine and Jean-Francois Remacle\n");
+        fprintf(stderr, "Copyright (C) 1997-2017 Christophe Geuzaine and Jean-Francois Remacle\n");
         PrintUsage(argv[0]);
         Msg::Exit(0);
       }
@@ -996,6 +1025,13 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing number");
       }
+      else if(!strcmp(argv[i] + 1, "nt")) {
+        i++;
+        if(argv[i])
+          Msg::SetNumThreads(atoi(argv[i++]));
+        else
+          Msg::Fatal("Missing number");
+      }
 #if defined(HAVE_FLTK)
       else if(!strcmp(argv[i] + 1, "term")) {
         terminal = 1;
@@ -1058,8 +1094,6 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "stereo")) {
         opt_general_stereo_mode(0, GMSH_SET, 1.);
-	//        CTX::instance()->camera = 1;
-	//	CTX::instance()->stereo = 1;
         i++;
       }
       else if(!strcmp(argv[i] + 1, "gamepad")) {
@@ -1129,3 +1163,5 @@ void GetOptions(int argc, char *argv[])
   if(CTX::instance()->terminal == 99)
     CTX::instance()->terminal = terminal;
 }
+
+
