@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -24,8 +24,8 @@ struct contextMeshOptions {
   int pyramids, trihedra;
   int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
   int pointsNum, linesNum, surfacesNum, volumesNum, qualityType, labelType;
-  int optimize, optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps;
-  double normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
+  int optimize,  optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps;
+  double optimizeThreshold,normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
   double mshFileVersion, mshFilePartitioned, pointSize, lineWidth;
   double qualityInf, qualitySup, radiusInf, radiusSup;
   double scalingFactor, lcFactor, randFactor, lcIntegrationPrecision;
@@ -49,7 +49,8 @@ struct contextMeshOptions {
   int minCircPoints, minCurvPoints;
   int hoOptimize, hoNLayers, hoOptPrimSurfMesh;
   double hoThresholdMin, hoThresholdMax, hoPoissonRatio;
-  int saveAll, saveTri, saveGroupsOfNodes, binary, bdfFieldFormat, saveParametric;
+  int saveAll, saveTri, saveGroupsOfNodes, binary, bdfFieldFormat;
+  int saveParametric, saveTopology;
   int smoothNormals, zoneDefinition, clip;
   int saveElementTagType;
   int switchElementTags;
@@ -60,6 +61,7 @@ struct contextMeshOptions {
   int ignorePartBound;
   int NewtonConvergenceTestXYZ;
   int preserveNumberingMsh2;
+  int ignorePeriodicity;
 };
 
 struct contextGeometryOptions {
@@ -71,13 +73,14 @@ struct contextGeometryOptions {
   int extrudeSplinePoints, extrudeReturnLateral;
   double normals, tangents, scalingFactor;
   int autoCoherence, highlightOrphans, clip, useTransform;
-  double tolerance, snap[3], transform[3][3], offset[3];
-  int occFixDegenerated, occFixSmallEdges, occFixSmallFaces;
-  int occSewFaces, occConnectFaces;
+  double tolerance, toleranceBoolean, snap[3], transform[3][3], offset[3];
+  int occAutoFix, occFixDegenerated, occFixSmallEdges, occFixSmallFaces;
+  int occSewFaces, occParallel, occBooleanPreserveNumbering;
   double occScaling;
-  int copyMeshingMethod, copyDisplayAttributes, exactExtrusion;
+  int copyMeshingMethod, exactExtrusion;
   int matchGeomAndMesh;
   int hideCompounds, orientedPhysicals, doubleClickedEntityTag;
+  int reparamOnFaceRobust;
   std::string doubleClickedPointCommand, doubleClickedLineCommand;
   std::string doubleClickedSurfaceCommand, doubleClickedVolumeCommand;
 };
@@ -137,7 +140,7 @@ class CTX {
   int highResolutionGraphics;
   // batch mode (-4: lua session, -3: server daemon, -2: check coherence, -1: write
   // geo, 0: full gfx, 1: 1D mesh, 2: 2D mesh, 3: 3D mesh, 4: adapt
-  // mesh, 5: refine mesh)
+  // mesh, 5: refine mesh, 6: reclassify mesh)
   int batch;
   // batch operations to apply after meshing (1: partition mesh)
   int batchAfterMesh;
@@ -145,6 +148,8 @@ class CTX {
   int meshDiscrete;
   // initial menu (0: automatic, 1: geom, 2: mesh, 3: solver, 4: post)
   int initialContext;
+  // show some windows on startup?
+  int showOptionsOnStartup, showMessagesOnStartup;
   // never popup dialogs in scripts (use default values instead)?
   int noPopup;
   // make all windows "non modal"?
@@ -274,7 +279,7 @@ class CTX {
     int jpegQuality, jpegSmoothing, geoLabels, geoOnlyPhysicals;
     int text, texAsEquation;
     int gifDither, gifSort, gifInterlace, gifTransparent;
-    int posElementary, posElement, posGamma, posEta, posRho, posDisto;
+    int posElementary, posElement, posGamma, posEta, posSICN, posSIGE, posDisto;
     int compositeWindows, deleteTmpFiles, background;
     int width, height;
     double parameter, parameterFirst, parameterLast, parameterSteps;
