@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -531,12 +531,12 @@ StringXNumber GeneralOptions_Number[] = {
   { F|S, "FltkColorScheme" , opt_general_gui_color_scheme , 0. ,
     "FLTK user interface color theme (0: standard, 1:dark)" },
   { F|O, "FontSize" , opt_general_fontsize , -1. ,
-    "Size of the font in the user interface (-1=automatic)" },
+    "Size of the font in the user interface, in pixels (-1: automatic)" },
 
   { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 15. ,
-    "Size of the font in the graphic window" },
+    "Size of the font in the graphic window, in pixels" },
   { F|O, "GraphicsFontSizeTitle" , opt_general_graphics_fontsize_title , 18. ,
-    "Size of the font in the graphic window for titles" },
+    "Size of the font in the graphic window for titles, in pixels" },
   { F|S, "GraphicsHeight" , opt_general_graphics_size1 , 600. ,
     "Height (in pixels) of the graphic window" },
   { F|S, "GraphicsPositionX" , opt_general_graphics_position0 , 50. ,
@@ -655,7 +655,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "MeshDiscrete" , opt_general_meshdiscrete , 0. ,
     "Mesh discrete surfaces through automatic parametrization (MUMPS required for efficiency) (0)" },
   { F|O, "MessageFontSize" , opt_general_message_fontsize , -1. ,
-    "Size of the font in the message window (-1=automatic)" },
+    "Size of the font in the message window, in pixels (-1: automatic)" },
   { F|S, "MessageHeight" , opt_general_message_size , 300. ,
     "Height (in pixels) of the message console when it is visible (should be > 0)" },
   { F,   "MinX" , opt_general_xmin , 0. ,
@@ -744,6 +744,10 @@ StringXNumber GeneralOptions_Number[] = {
     "Material shininess" },
   { F|O, "ShininessExponent" , opt_general_shine_exponent , 40. ,
     "Material shininess exponent (between 0 and 128)" },
+  { F|O, "ShowOptionsOnStartup", opt_general_show_options_on_startup, 0. ,
+    "Show option window on startup" },
+  { F|O, "ShowMessagesOnStartup", opt_general_show_messages_on_startup, 0. ,
+    "Show message window on startup" },
   { F|O, "SmallAxes" , opt_general_small_axes , 1. ,
     "Display the small axes" },
   { F|O, "SmallAxesPositionX" , opt_general_small_axes_position0 , -60. ,
@@ -814,9 +818,6 @@ StringXNumber GeometryOptions_Number[] = {
     "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
   { F|O, "CopyMeshingMethod" , opt_geometry_copy_meshing_method, 0. ,
     "Copy meshing method (unstructured or transfinite) when duplicating geometrical entities?" },
-  { F|O, "CopyDisplayAttributes" , opt_geometry_copy_display_attributes, 0. ,
-    "Copy display attributes (visibiliy, color) when duplicating geometrical entities?" },
-
   { F, "DoubleClickedEntityTag" , opt_geometry_double_clicked_entity_tag, 0. ,
     "Tag of last double-clicked geometrical entity" },
 
@@ -859,6 +860,9 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "NumSubEdges" , opt_geometry_num_sub_edges , 20. ,
     "Number of edge subdivisions between control points when displaying curves" },
 
+  { F|O, "OCCAutoFix" , opt_geometry_occ_auto_fix , 1. ,
+    "Automatically fix orientation of wires, faces, shells and volumes when creating"
+    " new entities" },
   { F|O, "OCCFixDegenerated" , opt_geometry_occ_fix_degenerated , 0. ,
     "Fix degenerated edges/faces in STEP, IGES and BRep models" },
   { F|O, "OCCFixSmallEdges" , opt_geometry_occ_fix_small_edges , 0. ,
@@ -867,8 +871,10 @@ StringXNumber GeometryOptions_Number[] = {
     "Fix small faces in STEP, IGES and BRep models" },
   { F|O, "OCCSewFaces" , opt_geometry_occ_sew_faces , 0. ,
     "Sew faces in STEP, IGES and BRep models" },
-  { F|O, "OCCConnectFaces" , opt_geometry_occ_connect_faces , 0. ,
-    "Cut and connect faces in STEP, IGES and BRep models (0=no, 1=internal, 2=Salome)" },
+  { F|O, "OCCParallel" , opt_geometry_occ_parallel , 0. ,
+    "Use multi-threaded OCC boolean operators" },
+  { F|O, "OCCBooleanPreserveNumbering" , opt_geometry_occ_boolean_preserve_numbering , 1. ,
+    "Try to preserve numbering of entities through OCC boolean operations" },
   { F|O, "OCCScaling" , opt_geometry_occ_scaling , 1. ,
     "Scale STEP, IGES and BRep model by given factor" },
   { F,   "OffsetX" , opt_geometry_offset0 , 0. ,
@@ -896,6 +902,10 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "PointType" , opt_geometry_point_type , 0. ,
     "Display points as solid color dots (0) or 3D spheres (1)" },
 
+  { F|O, "ReparamOnFaceRobust" , opt_geometry_reparam_on_face_robust, 0 ,
+    "Use projection for reparametrization of a point classified on GEdge on a GFace" },
+
+
   { F|O, "ScalingFactor" , opt_geometry_scaling_factor , 1.0 ,
     "Global geometry scaling factor" },
   { F|O, "OrientedPhysicals" , opt_geometry_oriented_physicals, 1. ,
@@ -910,13 +920,15 @@ StringXNumber GeometryOptions_Number[] = {
     "Display geometry surfaces?" },
   { F|O, "SurfaceNumbers" , opt_geometry_surfaces_num , 0. ,
     "Display surface numbers?" },
-  { F|O, "SurfaceType" , opt_geometry_surface_type , 2. ,
+  { F|O, "SurfaceType" , opt_geometry_surface_type , 0. ,
     "Surface display type (0=cross, 1=wireframe, 2=solid)" },
 
   { F|O, "Tangents" , opt_geometry_tangents , 0. ,
     "Display size of tangent vectors (in pixels)" },
   { F|O, "Tolerance" , opt_geometry_tolerance, 1.e-8 ,
     "Geometrical tolerance" },
+  { F|O, "ToleranceBoolean" , opt_geometry_tolerance_boolean, 0. ,
+    "Geometrical tolerance for boolean operations" },
   { F,   "Transform" , opt_geometry_transform , 0. ,
     "Transform model display coordinates (0=no, 1=scale)" },
   { F,   "TransformXX" , opt_geometry_transform00 , 1. ,
@@ -1152,9 +1164,11 @@ StringXNumber MeshOptions_Number[] = {
 
   { F|O, "OldRefinement" , opt_mesh_old_refinement , 1 ,
     "Use old 3D point insertion algorithm" },
-  { F|O, "Optimize" , opt_mesh_optimize , 0. ,
+  { F|O, "Optimize" , opt_mesh_optimize , 1. ,
     "Optimize the mesh to improve the quality of tetrahedral elements" },
-  { F|O, "OptimizeNetgen" , opt_mesh_optimize_netgen , 0. ,
+  { F|O, "OptimizeThreshold" , opt_mesh_optimize_threshold , 0.3 ,
+    "Optimize tetrahedra that have a quality below ... " },
+  { F|O, "OptimizeNetgen" , opt_mesh_optimize_netgen , 0 ,
     "Optimize the mesh using Netgen to improve the quality of tetrahedral "
     "elements" },
 
@@ -1182,8 +1196,9 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "QualitySup" , opt_mesh_quality_sup , 0.0 ,
     "Only display elements whose quality measure is smaller than QualitySup" },
   { F|O, "QualityType" , opt_mesh_quality_type , 2. ,
-    "Type of quality measure (0=gamma~vol/sum_face/max_edge, "
-    "1=eta~vol^(2/3)/sum_edge^2, 2=rho~min_edge/max_edge)" },
+    "Type of quality measure (0=SICN~signed inverse condition number, "
+    "1=SIGE~signed inverse gradient error, 2=gamma~vol/sum_face/max_edge, "
+    "3=Disto~minJ/maxJ"},
 
   { F|O, "RadiusInf" , opt_mesh_radius_inf , 0.0 ,
     "Only display elements whose longest edge is greater than RadiusInf" },
@@ -1197,6 +1212,9 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "PreserveNumberingMsh2" , opt_mesh_preserve_numbering_msh2 , 0. ,
     "Preserve element numbering in MSH2 format (will break meshes with multiple "
     "physical groups for a single elementary entity)"},
+  { F|O, "IgnorePeriodicity" , opt_mesh_ignore_periodicity , 0. ,
+    "Ignore alignement of periodic boundaries when reading the mesh "
+    "(used by ParaView plugin)"},
 #if defined(HAVE_BLOSSOM)
   { F|O, "RecombinationAlgorithm" , opt_mesh_algo_recombine , 1 ,
 #else
@@ -1232,6 +1250,8 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "SaveElementTagType" , opt_mesh_save_element_tag_type , 1. ,
     "Type of the element tag saved in mesh formats that don't support saving "
     "physical or partition ids (1=elementary, 2=physical, 3=partition)" },
+  { F|O, "SaveTopology" , opt_mesh_save_topology, 0. ,
+    "Save model topology in MSH2 output files (this is always saved in MSH3)" },
   { F|O, "SaveParametric" , opt_mesh_save_parametric , 0. ,
     "Save parametric coordinates of nodes" },
   { F|O, "SaveGroupsOfNodes" , opt_mesh_save_groups_of_nodes , 0. ,
@@ -1744,9 +1764,12 @@ StringXNumber PrintOptions_Number[] = {
   { F|O, "PostEta" , opt_print_pos_eta , 0. ,
     "Save Eta quality measure in mesh statistics exported as "
     "post-processing views" },
-  { F|O, "PostRho" , opt_print_pos_rho , 0. ,
-    "Save Rho quality measure in mesh statistics exported as "
-    "post-processing views" },
+  { F|O, "PostSICN" , opt_print_pos_SICN , 0. ,
+    "Save SICN (signed inverse condition number) quality measure in mesh "
+    "statistics exported as post-processing views" },
+  { F|O, "PostSICN" , opt_print_pos_SIGE , 0. ,
+    "Save SIGE (signed inverse gradient error) quality measure in mesh "
+    "statistics exported as post-processing views" },
   { F|O, "PostDisto" , opt_print_pos_disto , 0. ,
     "Save Disto quality measure in mesh statistics exported as "
     "post-processing views" },
