@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -52,7 +52,9 @@
 #define FORMAT_MPEG_PREVIEW 43
 #define FORMAT_PGF   44
 #define FORMAT_PVTU  45
-#define FORMAT_X3D  46
+#define FORMAT_X3D   46
+#define FORMAT_TOCHNOG 47
+#define FORMAT_TIKZ  48
 
 // Element types
 #define TYPE_PNT     1
