@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -10,6 +10,8 @@
 #elif defined(HAVE_NO_INTPTR_T)
 typedef unsigned long intptr_t;
 #endif
+#include <fstream>
+#include <streambuf>
 #include <ctype.h>
 #include "GmshMessage.h"
 #include "onelab.h"
@@ -528,6 +530,10 @@ void onelabGroup::_addParameter(T &p)
   Fl_Color c;
   if(getFlColor(p.getAttribute("Highlight"), c)) highlight = true;
   Fl_Tree_Item *n = _tree->add(p.getName().c_str());
+  if(!n){
+    Msg::Debug("Could not add item '%s' in tree", p.getName().c_str());
+    return;
+  }
   n->labelsize(FL_NORMAL_SIZE + 4);
   _tree->begin();
   int ww = (int)(_baseWidth - (n->depth() + 1) * _indent);
@@ -549,6 +555,10 @@ void onelabGroup::_addParameter(T &p)
 void onelabGroup::_addMenu(const std::string &path, Fl_Callback *callback, void *data)
 {
   Fl_Tree_Item *n = _tree->add(path.c_str());
+  if(!n){
+    Msg::Debug("Could not add item '%s' in tree", path.c_str());
+    return;
+  }
   _tree->begin();
   int ww = (int)(_baseWidth - (n->depth() + 1) * _indent);
   int hh = n->labelsize() + 4;
@@ -575,6 +585,10 @@ void onelabGroup::_addSolverMenu(int num)
   std::ostringstream path;
   path << "0Modules/Solver/Solver" << num;
   Fl_Tree_Item *n = _tree->add(path.str().c_str());
+  if(!n){
+    Msg::Debug("Could not add item '%s' in tree", path.str().c_str());
+    return;
+  }
   int ww = (int)(_baseWidth - (n->depth() + 1) * _indent);
   int hh = n->labelsize() + 4;
   _tree->begin();
@@ -592,6 +606,10 @@ void onelabGroup::_addViewMenu(int num)
   std::string path = getViewPathName(num);
   if(path.empty()) return;
   Fl_Tree_Item *n = _tree->add(path.c_str());
+  if(!n){
+    Msg::Debug("Could not add item '%s' in tree", path.c_str());
+    return;
+  }
   int ww = (int)(_baseWidth - (n->depth() + 1) * _indent);
   int hh = n->labelsize() + 4;
   _tree->begin();
@@ -1116,20 +1134,20 @@ void onelabGroup::rebuildTree(bool deleteWidgets)
   std::vector<onelab::number> numbers;
   onelab::server::instance()->get(numbers);
   for(unsigned int i = 0; i < numbers.size(); i++){
-    if(!numbers[i].getVisible() && !CTX::instance()->solver.showInvisibleParameters)
-      continue;
     if(numbers[i].getAttribute("Closed") == "1")
       closed.insert(numbers[i].getPath());
+    if(!numbers[i].getVisible() && !CTX::instance()->solver.showInvisibleParameters)
+      continue;
     _addParameter(numbers[i]);
   }
 
   std::vector<onelab::string> strings;
   onelab::server::instance()->get(strings);
   for(unsigned int i = 0; i < strings.size(); i++){
-    if(!strings[i].getVisible() && !CTX::instance()->solver.showInvisibleParameters)
-      continue;
     if(strings[i].getAttribute("Closed") == "1")
       closed.insert(strings[i].getPath());
+    if(!strings[i].getVisible() && !CTX::instance()->solver.showInvisibleParameters)
+      continue;
     _addParameter(strings[i]);
   }
 
@@ -1188,6 +1206,12 @@ void onelabGroup::rebuildTree(bool deleteWidgets)
     for(unsigned int i = 0; i < delStrings.size(); i++)
       free(delStrings[i]);
   }
+
+#if 0 // test
+  std::ifstream t("test.db.json");
+  std::string str((std::istreambuf_iterator<char>(t)), std::istreambuf_iterator<char>());
+  onelab::server::instance()->fromJSON(str);
+#endif
 }
 
 void onelabGroup::openTreeItem(const std::string &name)
@@ -1396,6 +1420,8 @@ void onelabGroup::addSolver(const std::string &name, const std::string &executab
   opt_solver_name(index, GMSH_SET, name);
   if(needToChooseExe(executable))
     onelab_choose_executable_cb(0, (void *)c);
+  else
+    opt_solver_executable(index, GMSH_SET, executable);
   opt_solver_remote_login(index, GMSH_SET, remoteLogin);
 
   FlGui::instance()->onelab->rebuildSolverList();
