@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -285,10 +285,12 @@ std::vector<MVertex*> GFace::getEmbeddedMeshVertices() const
   for(std::list<GEdge *>::const_iterator it = embedded_edges.begin();
       it != embedded_edges.end(); it++){
     tmp.insert((*it)->mesh_vertices.begin(), (*it)->mesh_vertices.end());
-    tmp.insert((*it)->getBeginVertex()->mesh_vertices.begin(),
-               (*it)->getBeginVertex()->mesh_vertices.end());
-    tmp.insert((*it)->getEndVertex()->mesh_vertices.begin(),
-               (*it)->getEndVertex()->mesh_vertices.end());
+    if((*it)->getBeginVertex())
+      tmp.insert((*it)->getBeginVertex()->mesh_vertices.begin(),
+                 (*it)->getBeginVertex()->mesh_vertices.end());
+    if((*it)->getEndVertex())
+      tmp.insert((*it)->getEndVertex()->mesh_vertices.begin(),
+                 (*it)->getEndVertex()->mesh_vertices.end());
   }
   for(std::list<GVertex *>::const_iterator it = embedded_vertices.begin();
       it != embedded_vertices.end(); it++){
@@ -323,6 +325,15 @@ void GFace::setVisibility(char val, bool recursive)
   }
 }
 
+void GFace::setColor(unsigned int val, bool recursive)
+{
+  GEntity::setColor(val);
+  if(recursive){
+    for (std::list<GEdge*>::iterator it = l_edges.begin(); it != l_edges.end(); ++it)
+      (*it)->setColor(val, recursive);
+  }
+}
+
 std::string GFace::getAdditionalInfoString()
 {
   std::ostringstream sstream;
@@ -374,7 +385,7 @@ void GFace::writeGEO(FILE *fp)
       fprintf(fp, "Plane Surface(%d) = {%d};\n", tag(), tag());
     }
     else if(edg.size() == 3 || edg.size() == 4){
-      fprintf(fp, "Ruled Surface(%d) = {%d};\n", tag(), tag());
+      fprintf(fp, "Surface(%d) = {%d};\n", tag(), tag());
     }
     else{
       Msg::Error("Skipping surface %d in export", tag());
@@ -441,7 +452,10 @@ void GFace::computeMeanPlane()
       for(unsigned int i = 2; i < pts.size(); i++){
         SVector3 d0i(pts[0], pts[i]);
         SVector3 n = crossprod(d01, d0i);
-        if(norm(n) > 1e-12){
+        // if too small, the points are almost colinear; tolerance is relatively
+        // high so that we don't accept points on plane surfaces defined by
+        // lines that are not exactly co-planar
+        if(norm(n) > sqrt(CTX::instance()->geom.tolerance) * CTX::instance()->lc){
           res[0] = n.x(); res[1] = n.y(); res[2] = n.z();
           xm = pts[0].x(); ym = pts[0].y(); zm = pts[0].z();
           ok = true;
@@ -513,6 +527,8 @@ void GFace::computeMeanPlane(const std::vector<MVertex*> &points)
 
 void GFace::computeMeanPlane(const std::vector<SPoint3> &points)
 {
+  if(points.empty()) return;
+
   // The concept of a mean plane computed in the sense of least
   // squares is fine for plane surfaces(!), but not really the best
   // one for non-plane surfaces. Indeed, imagine a quarter of a circle
@@ -683,7 +699,7 @@ void GFace::computeMeshSizeFieldAccuracy(double &avg,double &max_e, double &min_
   double oneoversqr2 = 1. / sqrt(2.);
   double sqr2 = sqrt(2.);
   for (std::set<MEdge, Less_Edge>::const_iterator it = es.begin();
-       it != es.end();++it){
+       it != es.end(); ++it){
     double u1,v1,u2,v2;
     MVertex *vert1 =  it->getVertex(0);
     vert1->getParameter(0, u1);
@@ -1316,7 +1332,7 @@ int GFace::genusGeom() const
   int nSeams = 0;
   std::set<GEdge*> single_seams;
   for (std::list<GEdge*>::const_iterator it = l_edges.begin();
-       it!=l_edges.end();++it){
+       it != l_edges.end(); ++it){
     if ((*it)->isSeam(this)){
       nSeams++;
       std::set<GEdge*>::iterator it2 = single_seams.find(*it);
@@ -1379,8 +1395,8 @@ bool GFace::fillPointCloud(double maxDist,
 
 
 #if defined(HAVE_MESH)
-static void meshCompound (GFace* gf, bool verbose) {
-
+static void meshCompound(GFace* gf, bool verbose)
+{
   discreteFace *df = new discreteFace (gf->model(), gf->tag() + 100000);
 
   std::set<int> ec;
@@ -1392,15 +1408,17 @@ static void meshCompound (GFace* gf, bool verbose) {
       if (found == ec.end())ec.insert((*it)->tag());
       else ec.erase(found);
     }
-    df->triangles.insert(df->triangles.end(), c->triangles.begin(),c->triangles.end());
-    df->mesh_vertices.insert(df->mesh_vertices.end(), c->mesh_vertices.begin(),c->mesh_vertices.end());
+    df->triangles.insert(df->triangles.end(), c->triangles.begin(),
+                         c->triangles.end());
+    df->mesh_vertices.insert(df->mesh_vertices.end(), c->mesh_vertices.begin(),
+                             c->mesh_vertices.end());
     for (unsigned int j=0;j<c->triangles.size();j++)df->_CAD.push_back(c);
     c->triangles.clear();
     c->mesh_vertices.clear();
   }
   std::vector<int> cedges;
   cedges.insert(cedges.begin(), ec.begin(), ec.end());
-  df->setBoundEdges(gf->model(), cedges);
+  df->setBoundEdges(cedges);
   df->createGeometry();
   df->mesh(verbose);
   delete df;
@@ -1415,7 +1433,7 @@ void GFace::mesh(bool verbose)
   if (!_compound.empty()){ // Some faces are meshed together
     if (_compound[0] == this){ //  I'm the one that makes the compound job
       bool ok = true;
-      for (unsigned int i=0;i<_compound.size();i++){
+      for (unsigned int i = 0; i < _compound.size(); i++){
 	GFace *gf = (GFace*)_compound[i];
 	ok &= (gf->meshStatistics.status == GFace::DONE);
       }
@@ -1505,7 +1523,7 @@ void GFace::setMeshMaster(GFace* master, const std::vector<double>& tfo)
   std::set<GVertex*> l_vertices;
   std::map<std::pair<GVertex*,GVertex*>,GEdge* > l_vtxToEdge;
 
-  for (eIter=l_edges.begin();eIter!=l_edges.end();++eIter) {
+  for (eIter=l_edges.begin();eIter!=l_edges.end(); ++eIter) {
     GVertex* v0 = (*eIter)->getBeginVertex();
     GVertex* v1 = (*eIter)->getEndVertex();
     l_vertices.insert(v0);
@@ -1513,7 +1531,7 @@ void GFace::setMeshMaster(GFace* master, const std::vector<double>& tfo)
     l_vtxToEdge[std::make_pair(v0,v1)] = (*eIter);
   }
 
-  for (eIter=embedded_edges.begin();eIter!=embedded_edges.end();++eIter) {
+  for (eIter=embedded_edges.begin();eIter!=embedded_edges.end(); ++eIter) {
     GVertex* v0 = (*eIter)->getBeginVertex();
     GVertex* v1 = (*eIter)->getEndVertex();
     l_vertices.insert(v0);
@@ -1528,7 +1546,7 @@ void GFace::setMeshMaster(GFace* master, const std::vector<double>& tfo)
   std::list<GEdge*> m_edges = master->edges();
   std::set<GVertex*> m_vertices;
   std::map<std::pair<GVertex*,GVertex*>,GEdge* > m_vtxToEdge;
-  for (eIter=m_edges.begin();eIter!=m_edges.end();++eIter) {
+  for (eIter=m_edges.begin();eIter!=m_edges.end(); ++eIter) {
     GVertex* v0 = (*eIter)->getBeginVertex();
     GVertex* v1 = (*eIter)->getEndVertex();
     m_vertices.insert(v0);
@@ -1570,7 +1588,7 @@ void GFace::setMeshMaster(GFace* master, const std::vector<double>& tfo)
   std::map<GVertex*,GVertex*> gVertexCounterparts;
 
   std::set<GVertex*>::iterator mvIter;
-  for (mvIter=m_vertices.begin();mvIter!=m_vertices.end();++mvIter) {
+  for (mvIter=m_vertices.begin();mvIter!=m_vertices.end(); ++mvIter) {
 
     GVertex* m_vertex = *mvIter;
 
@@ -1585,12 +1603,13 @@ void GFace::setMeshMaster(GFace* master, const std::vector<double>& tfo)
 
     GVertex* l_vertex = NULL;
 
+    double dist_min = 1.e22;
     std::set<GVertex*>::iterator lvIter = l_vertices.begin();
-    for (;lvIter!=l_vertices.end();++lvIter) {
+    for (;lvIter!=l_vertices.end(); ++lvIter) {
 
       SPoint3 xyz((*lvIter)->x(),(*lvIter)->y(),(*lvIter)->z());
       SVector3 dist = xyz - xyzTfo;
-
+      dist_min = std::min(dist_min,dist.norm());
       if (dist.norm() < CTX::instance()->geom.tolerance * CTX::instance()->lc) {
         l_vertex = *lvIter;
         break;
@@ -1600,8 +1619,9 @@ void GFace::setMeshMaster(GFace* master, const std::vector<double>& tfo)
     if (l_vertex==NULL) {
       Msg::Error("Was not able to find corresponding node %d "
                  "for periodic connection of surface %d to %d "
-                 "using the specified transformation",
-                 m_vertex->tag(),master->tag(),tag());
+                 "using the specified transformation"
+                 "Minimum distance is %g with a tolerance of %g",
+                 m_vertex->tag(),master->tag(),tag(),dist_min, CTX::instance()->geom.tolerance * CTX::instance()->lc);
       return;
     }
     gVertexCounterparts[l_vertex] = m_vertex;
@@ -1732,8 +1752,7 @@ void GFace::setMeshMaster(GFace* master,const std::map<int,int>& edgeCopies)
 {
   std::map<GVertex*,GVertex*> vs2vt;
 
-  for (std::list<GEdge*>::iterator it=l_edges.begin();it!=l_edges.end();++it){
-
+  for (std::list<GEdge*>::iterator it = l_edges.begin(); it != l_edges.end(); ++it){
     // slave edge
     GEdge* le = *it;
 
