@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -23,7 +23,6 @@ template <class scalar> class simpleFunction;
 class FM_Internals;
 class GEO_Internals;
 class OCC_Internals;
-class SGEOM_Internals;
 class ACIS_Internals;
 class smooth_normals;
 class FieldManager;
@@ -35,11 +34,9 @@ class MElementOctree;
 class GModelFactory;
 
 // A geometric model. The model is a "not yet" non-manifold B-Rep.
-class GModel
-{
+class GModel {
  private:
   friend class OCCFactory;
-  friend class SGEOMFactory;
   std::multimap<std::pair<std::vector<int>, std::vector<int> >,
                 std::pair<std::string, std::vector<int> > > _homologyRequests;
   std::set<GRegion*, GEntityLessThan> _chainRegions;
@@ -94,10 +91,7 @@ class GModel
   // OpenCascade model internal data
   OCC_Internals *_occ_internals;
   void _deleteOCCInternals();
-
-  // SGEOM model internal data
-  SGEOM_Internals *_sgeom_internals;
-  void _deleteSGEOMInternals();
+  void _resetOCCInternals();
 
   // ACIS model internal data
   ACIS_Internals *_acis_internals;
@@ -214,8 +208,8 @@ class GModel
 
   // access internal CAD representations
   GEO_Internals *getGEOInternals(){ return _geo_internals; }
+  void createOCCInternals();
   OCC_Internals *getOCCInternals(){ return _occ_internals; }
-  SGEOM_Internals *getSGEOMInternals(){ return _sgeom_internals; }
   FM_Internals *getFMInternals() { return _fm_internals; }
   ACIS_Internals *getACISInternals(){ return _acis_internals; }
 
@@ -272,7 +266,7 @@ class GModel
   GFace *getFaceByTag(int n) const;
   GEdge *getEdgeByTag(int n) const;
   GVertex *getVertexByTag(int n) const;
-  std::vector<int> getEdgesByStringTag(const std::string tag);
+  GEntity *getEntityByTag(int dim, int n) const;
 
   // add/remove an entity in the model
   void add(GRegion *r) { regions.insert(r); }
@@ -283,6 +277,8 @@ class GModel
   void remove(GFace *f);
   void remove(GEdge *e);
   void remove(GVertex *v);
+  void remove(int dim, int tag, bool recursive=false);
+  void remove(const std::vector<std::pair<int, int> > &dimTags, bool recursive=false);
 
   // snap vertices on model edges by using geometry tolerance
   void snapVertices();
@@ -294,6 +290,11 @@ class GModel
   void getEntitiesInBox(std::vector<GEntity*> &entities, SBoundingBox3d box,
                         int dim=-1) const;
 
+  // get tags of entities of the boundary of the given input entities
+  bool getBoundaryTags(const std::vector<std::pair<int, int> > &inDimTags,
+                       std::vector<std::pair<int, int> > &outDimTags,
+                       bool combined, bool oriented=true, bool recursive=false);
+
   // return the highest number associated with an elementary entity of
   // a given dimension (or the highest overall if dim < 0)
   int getMaxElementaryNumber(int dim);
@@ -337,6 +338,15 @@ class GModel
   // "dim" and name "name". return -1 if not found
   int getPhysicalNumber(const int &dim, const std::string & name);
 
+  // get all tags of elementary entities associated with a given physical group
+  // name
+  std::vector<int> getTagsForPhysicalName(int dim, const std::string name);
+  // deprecated
+  std::vector<int> getEdgesByStringTag(const std::string name)
+  {
+    return getTagsForPhysicalName(1, name);
+  }
+
   // set physical tags to entities in a given bounding box
   void setPhysicalNumToEntitiesInBox(int EntityDimension, int PhysicalNumber,
                                      std::vector<double> p1, std::vector<double> p2);
@@ -441,44 +451,21 @@ class GModel
   // mesh the model
   int mesh(int dimension);
 
-  // adapt the mesh anisotropically using metrics that are computed from a set of functions f(x,y,z).
-  //   For all algorithms
-  //           parameters[1] = lcmin (default : in global gmsh options CTX::instance()->mesh.lcMin)
-  //           parameters[2] = lcmax (default : in global gmsh options CTX::instance()->mesh.lcMax)
-  //   niter is the maximum number of iterations
-  //   Available algorithms:
-  //    1) Assume that the function is a levelset -> adapt using Coupez technique (technique = 1)
-  //           parameters[0] = thickness of the interface (mandatory)
-  //    2) Assume that the function is a physical quantity -> adapt using the Hessian (technique = 2)
-  //           parameters[0] = N, the final number of elements
-  //    3) A variant of 1) by P. Frey (= Coupez + takes curvature function into account)
-  //           parameters[0] = thickness of the interface (mandatory)
-  //           parameters[3] = the required minimum number of elements to represent a circle - used for curvature-based metric (default: = 15)
-  //    4) A variant (3), direct implementation in the metric eigendirections, assuming a level set (ls):
-  //        - hmin is imposed in the ls gradient,
-  //        - hmax is imposed in the two eigendirections of the ls hessian that are (almost ?) tangent to the iso-zero plane
-  //          + the latter eigenvalues (1/hmax^2) are modified if necessary to capture the iso-zero curvature
-  //           parameters[0] = thickness of the interface in the positive ls direction (mandatory)
-  //           parameters[4] = thickness of the interface in the negative ls direction (=parameters[0] if not specified)
-  //           parameters[3] = the required minimum number of elements to represent a circle - used for curvature-based metric (default: = 15)
-  //    5) Same as 4, except that the transition in band E uses linear interpolation of h, instead of linear interpolation of metric
-  // The algorithm first generate a mesh if no one is available
-
-  // In this first attempt, only the highest dimensional mesh is adapted, which is ok if
-  // we assume that boundaries are already adapted.
-  // This should be fixed.
+  // adapt the mesh anisotropically using metrics that are computed from a set
+  // of functions f(x,y,z). The algorithm first generate a mesh if no one is
+  // available; see the cpp for parameter documentation
   int adaptMesh(std::vector<int> technique,
 		std::vector<simpleFunction<double>*> f,
 		std::vector<std::vector<double> > parameters,
 		int niter, bool meshAll=false);
 
-  // Ensure that the Jacobian of all volume elements is positive
+  // ensure that the Jacobian of all volume elements is positive
   bool setAllVolumesPositive();
   void setAllVolumesPositiveTopology();
 
-  // make the mesh a high order mesh at order N
-  // linear is 1 if the high order points are not placed on the geometry of the model
-  // incomplete is 1 if incomplete basis are used
+  // make the mesh a high order mesh at order N (linear is 1 if the high order
+  // points are not placed on the geometry of the model; incomplete is 1 if
+  // incomplete basis are used)
   int setOrderN(int order, int linear, int incomplete);
 
   // refine the mesh by splitting all elements
@@ -499,6 +486,9 @@ class GModel
   // reclassify a mesh i.e. use an angle threshold to tag edges faces and regions
   void classifyFaces(std::set<GFace*> &_faces);
 
+  // classify a mesh for all faces on the current model
+  void classifyAllFaces();
+
   // glue entities in the model (assume a tolerance eps and merge
   // vertices that are too close, then merge edges, faces and
   // regions). Warning: the gluer changes the geometric model, so that
@@ -526,7 +516,8 @@ class GModel
   GEntity *extrude(GEntity *e, std::vector<double> p1, std::vector<double> p2);
   std::vector<GEntity*> extrudeBoundaryLayer(GEntity *e, int nbLayers, double hLayers,
                                              int dir=1, int view=-1);
-  GEntity *addPipe(GEntity *e, std::vector<GEdge *>  edges);
+  GEntity *addPipe(GEntity *e, std::vector<GEdge *> edges);
+  GEntity *addThruSections(std::vector<std::vector<GEdge *> > edges);
 
   std::vector<GFace *> addRuledFaces(std::vector<std::vector<GEdge *> > edges);
   GFace *addFace(std::vector<GEdge *> edges, std::vector< std::vector<double > > points);
@@ -556,8 +547,6 @@ class GModel
   GModel *computeBooleanUnion(GModel *tool, int createNewModel=0);
   GModel *computeBooleanIntersection(GModel *tool, int createNewModel=0);
   GModel *computeBooleanDifference(GModel *tool, int createNewModel=0);
-  void    salomeconnect();
-  void    occconnect();
 
   void setPeriodicAllFaces(std::vector<double> FaceTranslationVector);
   void setPeriodicPairOfFaces(int numFaceMaster, std::vector<int> EdgeListMaster,
@@ -609,7 +598,6 @@ class GModel
   static int readGEO(const std::string &name);
   int writeGEO(const std::string &name, bool printLabels=true,
                bool onlyPhysicals=false);
-  int importGEOInternals();
   int exportDiscreteGEOInternals();
 
   // Fourier model
@@ -648,8 +636,9 @@ class GModel
 
   // mesh statistics (saved as a Gmsh post-processing view)
   int writePOS(const std::string &name, bool printElementary,
-               bool printElementNumber, bool printSICN, bool printGamma, bool printRho,
-               bool printDisto, bool saveAll=false, double scalingFactor=1.0);
+               bool printElementNumber, bool printSICN, bool printSIGE,
+               bool printGamma, bool printDisto, bool saveAll=false,
+               double scalingFactor=1.0);
 
   // Stereo lithography format
   int readSTL(const std::string &name, double tolerance=1.e-3);
@@ -684,6 +673,9 @@ class GModel
   // Actran mesh
   int readACTRAN(const std::string &name);
 
+  // Sameced mesh
+  int readSAMCEF(const std::string &name);
+
   // Plot3D structured mesh format
   int readP3D(const std::string &name);
   int writeP3D(const std::string &name,
