@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -24,12 +24,11 @@ class GModelFactory {
  public:
   GModelFactory(){}
   virtual ~GModelFactory(){}
-
+  virtual std::string getName(){ return ""; }
   // brep primitives
   enum arcCreationMethod {THREE_POINTS=1, CENTER_START_END=2};
   enum splineType {BEZIER=1, BSPLINE=2};
-  virtual GVertex *addVertex(GModel *gm, double x, double y, double z,
-                             double lc) = 0;
+  virtual GVertex *addVertex(GModel *gm, double x, double y, double z, double lc) = 0;
   virtual GEdge *addLine(GModel *, GVertex *v1, GVertex *v2) = 0;
   virtual GFace *addPlanarFace(GModel *gm, std::vector<std::vector<GEdge *> > edges) = 0;
   virtual GFace *addPlanarFace(GModel *gm, const std::vector<std::vector<GEdgeSigned> > &edges) = 0;
@@ -55,10 +54,10 @@ class GModelFactory {
     return 0;
   }
   virtual GEdge *addNURBS(GModel *gm, GVertex *start, GVertex *end,
-			  std::vector<std::vector<double> > controlPoints,
-			  std::vector<double> knots,
-			  std::vector<double> weights,
-			  std::vector<int> multiplicity)
+        std::vector<std::vector<double> > controlPoints,
+        std::vector<double> knots,
+        std::vector<double> weights,
+        std::vector<int> multiplicity)
   {
     Msg::Error("addNURBS not implemented yet");
     return 0;
@@ -67,7 +66,7 @@ class GModelFactory {
   // faces. If boundaries are co-planar, then it's a plane, otherwise,
   // we tru ruled, sweep or other kind of surfaces
   virtual std::vector<GFace *> addRuledFaces(GModel *gm,
-					     std::vector<std::vector<GEdge *> > edges)
+               std::vector<std::vector<GEdge *> > edges)
   {
     Msg::Error("addRuledFaces not implemented yet");
     return std::vector<GFace*>();
@@ -115,6 +114,11 @@ class GModelFactory {
     Msg::Error("addPipe not implemented yet");
     return 0;
   }
+  virtual GEntity *addThruSections(GModel *gm, std::vector<std::vector<GEdge *> > wire)
+  {
+    Msg::Error("addThruSections not implemented yet");
+    return 0;
+  }
 
   // solid primitives
   virtual GEntity *addSphere(GModel *gm, double cx, double cy, double cz,
@@ -193,14 +197,6 @@ class GModelFactory {
     return 0;
   }
 
-	virtual void salomeconnect(GModel *gm)
-  {
-    Msg::Error("salomeconnect not implemented yet");
-  }
-  virtual void occconnect(GModel *gm)
-  {
-    Msg::Error("occconnect not implemented yet");
-  }
   virtual void setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTranslationVector)
   {
     Msg::Error("findperiodic not implemented yet");
@@ -218,6 +214,7 @@ class GModelFactory {
 class GeoFactory : public GModelFactory {
  public:
   GeoFactory(){}
+  std::string getName(){ return "GEO"; }
   GVertex *addVertex(GModel *gm,double x, double y, double z, double lc);
   GEdge *addLine(GModel *gm,GVertex *v1, GVertex *v2);
   GFace *addPlanarFace(GModel *gm, std::vector<std::vector<GEdge *> > edges);
@@ -238,6 +235,7 @@ private:
 class OCCFactory : public GModelFactory {
  public:
   OCCFactory(){}
+  std::string getName(){ return "OpenCASCADE"; }
   GVertex *addVertex(GModel *gm,double x, double y, double z, double lc);
   GEdge *addLine(GModel *gm,GVertex *v1, GVertex *v2);
   GEdge *addCircleArc(GModel *gm,const arcCreationMethod &method,
@@ -247,16 +245,17 @@ class OCCFactory : public GModelFactory {
                    GVertex *start, GVertex *end,
                    std::vector<std::vector<double> > controlPoints);
   GEdge *addNURBS(GModel *gm,
-		  GVertex *start, GVertex *end,
-		  std::vector<std::vector<double> > controlPoints,
-		  std::vector<double> knots,
-		  std::vector<double> weights,
-		  std::vector<int> multiplicity);
+      GVertex *start, GVertex *end,
+      std::vector<std::vector<double> > controlPoints,
+      std::vector<double> knots,
+      std::vector<double> weights,
+      std::vector<int> multiplicity);
   GEntity *revolve(GModel *gm, GEntity*,std::vector<double> p1,
                    std::vector<double> p2, double angle);
   GEntity *extrude(GModel *gm, GEntity*,std::vector<double> p1,
                    std::vector<double> p2);
   GEntity *addPipe(GModel *gm, GEntity *base, std::vector<GEdge *> wire);
+  GEntity *addThruSections(GModel *gm, std::vector<std::vector<GEdge *> > wire);
   GEntity *addSphere(GModel *gm,double cx, double cy, double cz, double radius);
   GEntity *addCylinder(GModel *gm,std::vector<double> p1, std::vector<double> p2,
                        double radius);
@@ -277,11 +276,11 @@ class OCCFactory : public GModelFactory {
   void translate(GModel *gm, std::vector<double> dx, int addToTheModel);
   void rotate(GModel *gm, std::vector<double> p1,std::vector<double> p2,
               double angle, int addToTheModel);
+  void dilate(GModel *gm, std::vector<double> p, std::vector<double> s, 
+              int addToTheModel);
   GModel *computeBooleanUnion(GModel *obj, GModel *tool, int createNewModel);
   GModel *computeBooleanIntersection(GModel *obj, GModel *tool, int createNewModel);
   GModel *computeBooleanDifference(GModel *obj, GModel *tool, int createNewModel);
-  void    salomeconnect(GModel *gm);
-  void    occconnect(GModel *gm);
 
   void setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTranslationVector);
   void setPeriodicPairOfFaces(GModel *gm, int numFaceMaster, std::vector<int> EdgeListMaster,
@@ -292,20 +291,4 @@ class OCCFactory : public GModelFactory {
 
 #endif
 
-#if defined(HAVE_SGEOM) && defined(HAVE_OCC)
-
-class SGEOMFactory : public GModelFactory {
- public:
-  SGEOMFactory(){}
-  GVertex *addVertex(GModel *gm,double x, double y, double z, double lc);
-  GEdge *addLine(GModel *gm,GVertex *v1, GVertex *v2);
-  GFace *addPlanarFace(GModel *gm, std::vector<std::vector<GEdge *> > edges);
-  GFace *addPlanarFace(GModel *gm, const std::vector<std::vector<GEdgeSigned> > &edges);
-  GRegion *addVolume(GModel *gm, std::vector<std::vector<GFace *> > faces);
-  void healGeometry(GModel *gm, double tolerance = -1.);
-};
-
-#endif
-
-
 #endif
