@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -116,16 +116,6 @@ double MElement::maxEdge()
   return m;
 }
 
-double MElement::rhoShapeMeasure()
-{
-  double min = minEdge();
-  double max = maxEdge();
-  if(max)
-    return min / max;
-  else
-    return 0.;
-}
-
 double MElement::maxDistToStraight() const
 {
   const nodalBasis *lagBasis = getFunctionSpace();
@@ -152,7 +142,7 @@ double MElement::maxDistToStraight() const
 double MElement::minIsotropyMeasure(bool knownValid, bool reversedOK)
 {
 #if defined(HAVE_MESH)
-  return jacobianBasedQuality::minIsotropyMeasure(this, knownValid, reversedOK);
+  return jacobianBasedQuality::minICNMeasure(this, knownValid, reversedOK);
 #else
   return 0.;
 #endif
@@ -161,7 +151,7 @@ double MElement::minIsotropyMeasure(bool knownValid, bool reversedOK)
 double MElement::minScaledJacobian(bool knownValid, bool reversedOK)
 {
 #if defined(HAVE_MESH)
-  return jacobianBasedQuality::minScaledJacobian(this, knownValid, reversedOK);
+  return jacobianBasedQuality::minIGEMeasure(this, knownValid, reversedOK);
 #else
   return 0.;
 #endif
@@ -175,7 +165,7 @@ double MElement::specialQuality()
   if (minJ <= 0.) return minJ;
 //  if (minJ < 0 && maxJ >= 0) return minJ/maxJ; // accept -inf as an answer
 //  if (minJ < 0 && maxJ < 0) return -std::numeric_limits<double>::infinity();
-  return jacobianBasedQuality::minIsotropyMeasure(this, true);
+  return jacobianBasedQuality::minICNMeasure(this, true);
 #else
   return 0;
 #endif
@@ -189,7 +179,7 @@ double MElement::specialQuality2()
   if (minJ <= 0.) return minJ;
 //  if (minJ < 0 && maxJ >= 0) return minJ/maxJ; // accept -inf as an answer
 //  if (minJ < 0 && maxJ < 0) return -std::numeric_limits<double>::infinity();
-  return jacobianBasedQuality::minScaledJacobian(this, true);
+  return jacobianBasedQuality::minIGEMeasure(this, true);
 #else
   return 0;
 #endif
@@ -334,6 +324,14 @@ void MElement::signedInvCondNumRange(double &iCNMin, double &iCNMax, GEntity *ge
 #endif
 }
 
+void MElement::signedInvGradErrorRange(double &minSIGE, double &maxSIGE)
+{
+#if defined(HAVE_MESH)
+  jacobianBasedQuality::sampleIGEMeasure(this, getPolynomialOrder(),
+                                         minSIGE, maxSIGE);
+#endif
+}
+
 void MElement::getNode(int num, double &u, double &v, double &w) const
 {
   // only for MElements that don't have a lookup table for this
@@ -780,9 +778,9 @@ void MElement::xyz2uvw(double xyz[3], double uvw[3]) const
     }
     const nodalBasis *nb = getFunctionSpace();
     fullMatrix<double> refpnts = nb->getReferenceNodes();
-    uvw[0] = refpnts(numNearer, 0);
-    uvw[1] = refpnts(numNearer, 1);
-    uvw[2] = refpnts(numNearer, 2);
+    for (int i=0; i < getDim(); i++){
+      uvw[i] = refpnts(numNearer, i);
+    }
   }
 
   int iter = 1, maxiter = 20;
@@ -1135,7 +1133,7 @@ void MElement::writeMSH2(FILE *fp, double version, bool binary, int num,
 }
 
 void MElement::writePOS(FILE *fp, bool printElementary, bool printElementNumber,
-                        bool printSICN, bool printGamma, bool printRho,
+                        bool printSICN, bool printSIGE, bool printGamma,
                         bool printDisto, double scalingFactor, int elementary)
 {
   const char *str = getStringForPOS();
@@ -1169,6 +1167,13 @@ void MElement::writePOS(FILE *fp, bool printElementary, bool printElementNumber,
       fprintf(fp, "%g", sICNMin);
     }
   }
+  if(printSIGE){
+    double sIGEMin = minSIGEShapeMeasure();
+    for(int i = 0; i < n; i++){
+      if(first) first = false; else fprintf(fp, ",");
+      fprintf(fp, "%g", sIGEMin);
+    }
+  }
   if(printGamma){
     double gamma = gammaShapeMeasure();
     for(int i = 0; i < n; i++){
@@ -1177,13 +1182,6 @@ void MElement::writePOS(FILE *fp, bool printElementary, bool printElementNumber,
       //fprintf(fp, "%d", getVertex(i)->getNum());
     }
   }
-  if(printRho){
-    double rho = rhoShapeMeasure();
-    for(int i = 0; i < n; i++){
-      if(first) first = false; else fprintf(fp, ",");
-      fprintf(fp, "%g", rho);
-    }
-  }
   if(printDisto){
     double disto = distoShapeMeasure();
     for(int i = 0; i < n; i++){
