@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -199,21 +199,26 @@ class MElement
 
   // get the quality measures
   double skewness();
-  virtual double rhoShapeMeasure();
   virtual double gammaShapeMeasure(){ return 0.; }
   virtual double etaShapeMeasure(){ return 0.; }
-  double distoShapeMeasure()
-  {
-    double jmin, jmax;
-    scaledJacRange(jmin, jmax);
-    return jmin;
-  }
   double minSICNShapeMeasure()
   {
     double sICNMin, sICNMax;
     signedInvCondNumRange(sICNMin, sICNMax);
     return sICNMin;
   }
+  double minSIGEShapeMeasure()
+  {
+    double minSIGE, maxSIGE;
+    signedInvGradErrorRange(minSIGE, maxSIGE);
+    return minSIGE;
+  }
+  double distoShapeMeasure()
+  {
+    double jmin, jmax;
+    scaledJacRange(jmin, jmax);
+    return jmin;
+  }
   double minIsotropyMeasure(bool knownValid = false, bool reversedOk = false);
   double minScaledJacobian(bool knownValid = false, bool reversedOk = false);
   double specialQuality();
@@ -222,6 +227,7 @@ class MElement
   virtual void scaledJacRange(double &jmin, double &jmax, GEntity *ge = 0) const;
   virtual void idealJacRange(double &jmin, double &jmax, GEntity *ge = 0);
   virtual void signedInvCondNumRange(double &iCNMin, double &iCNMax, GEntity *ge = 0);
+  virtual void signedInvGradErrorRange(double &minSIGE, double &maxSIGE);
 
   // get the radius of the inscribed circle/sphere if it exists,
   // otherwise get the minimum radius of all the circles/spheres
@@ -369,7 +375,7 @@ class MElement
                          int parentNum=0, int dom1Num = 0, int dom2Num = 0,
                          std::vector<short> *ghosts=0);
   virtual void writePOS(FILE *fp, bool printElementary, bool printElementNumber,
-                        bool printSICN, bool printGamma, bool printRho,
+                        bool printSICN, bool printSIGE, bool printGamma,
                         bool printDisto,double scalingFactor=1.0, int elementary=1);
   virtual void writeSTL(FILE *fp, bool binary=false, double scalingFactor=1.0);
   virtual void writeVRML(FILE *fp);
@@ -458,4 +464,11 @@ template <> struct DimTr<3>
   }
 };
 
+struct Less_ElementPtr : public std::binary_function<MElement *, MElement *, bool> {
+  bool operator()(const MElement *e1, const MElement *e2) const
+  {
+    return e1->getNum()<e2->getNum();
+  }
+};
+
 #endif
