@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -7,13 +7,15 @@
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GModel.h"
+#include "GModelIO_OCC.h"
 #include "OCCEdge.h"
 #include "OCCFace.h"
 #include "Context.h"
 
 #if defined(HAVE_OCC)
-#include "GModelIO_OCC.h"
+
 #include <Standard_Version.hxx>
+#include <TopoDS.hxx>
 #include <Geom2dLProp_CLProps2d.hxx>
 #include <Geom_BezierCurve.hxx>
 #include <Geom_OffsetCurve.hxx>
@@ -25,31 +27,47 @@
 #include <Geom_Line.hxx>
 #include <Geom_Conic.hxx>
 #include <Geom_BSplineCurve.hxx>
-#if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR < 6)
-#include <BOPTools_Tools.hxx>
-#else
+#include <GeomAPI_ProjectPointOnCurve.hxx>
+#include <Bnd_Box.hxx>
+#include <BRepLProp_CLProps.hxx>
+#include <BRepBndLib.hxx>
+#include <BRepAdaptor_Curve.hxx>
+#include <BRepAdaptor_Surface.hxx>
+#include <BRep_Builder.hxx>
 #include <BOPTools_AlgoTools.hxx>
-#endif
 
 OCCEdge::OCCEdge(GModel *m, TopoDS_Edge edge, int num, GVertex *v1, GVertex *v2)
   : GEdge(m, num, v1, v2), c(edge), trimmed(0)
 {
+  // force orientation of internal/external edges: otherwise reverse will not
+  // produce the expected result
+  if (c.Orientation() == TopAbs_INTERNAL || c.Orientation() == TopAbs_EXTERNAL){
+    c = TopoDS::Edge(c.Oriented(TopAbs_FORWARD));
+  }
   curve = BRep_Tool::Curve(c, s0, s1);
   // build the reverse curve
   c_rev = c;
   c_rev.Reverse();
-  model()->getOCCInternals()->bind(c, num);
+  if(model()->getOCCInternals())
+    model()->getOCCInternals()->bind(c, num);
 }
 
 OCCEdge::~OCCEdge()
 {
-  model()->getOCCInternals()->unbind(c);
+  if(model()->getOCCInternals())
+    model()->getOCCInternals()->unbind(c, tag());
 }
 
 SBoundingBox3d OCCEdge::bounds() const
 {
   Bnd_Box b;
-  BRepBndLib::Add(c, b);
+  try{
+    BRepBndLib::Add(c, b);
+  }
+  catch(Standard_Failure &err){
+    Msg::Error("OpenCASCADE exception %s", err.GetMessageString());
+    return SBoundingBox3d();
+  }
   double xmin, ymin, zmin, xmax, ymax, zmax;
   b.Get(xmin, ymin, zmin, xmax, ymax, zmax);
   SBoundingBox3d bbox(xmin, ymin, zmin, xmax, ymax, zmax);
@@ -73,7 +91,6 @@ void OCCEdge::setTrimmed(OCCFace *f)
 
 SPoint2 OCCEdge::reparamOnFace(const GFace *face, double epar, int dir) const
 {
-
   if (face->getNativeType() != GEntity::OpenCascadeModel){
     const GPoint pt = point(epar);
     SPoint3 sp(pt.x(), pt.y(), pt.z());
@@ -104,19 +121,23 @@ SPoint2 OCCEdge::reparamOnFace(const GFace *face, double epar, int dir) const
     pnt.Coord(u, v);
 
     // sometimes OCC miserably fails ...
-    if (0){
+    if (CTX::instance()->geom.reparamOnFaceRobust){
       GPoint p1 = point(epar);
       GPoint p2 = face->point(u, v);
       const double dx = p1.x()-p2.x();
       const double dy = p1.y()-p2.y();
       const double dz = p1.z()-p2.z();
-      if(sqrt(dx * dx + dy * dy + dz * dz) > 1.e-2 * CTX::instance()->lc){
+      if(sqrt(dx * dx + dy * dy + dz * dz) > CTX::instance()->geom.tolerance){
 	Msg::Warning("Reparam on face was inaccurate for curve %d on surface %d at point %g",
 		     tag(), face->tag(), epar);
 	Msg::Warning("On the face %d local (%g %g) global (%g %g %g)",
 		     face->tag(), u, v, p2.x(), p2.y(), p2.z());
 	Msg::Warning("On the edge %d local (%g) global (%g %g %g)",
 		     tag(), epar, p1.x(), p1.y(), p1.z());
+	double guess [2] =  {u,v};
+	GPoint pp = face->closestPoint(SPoint3(p1.x(),p1.y(),p1.z()), guess);
+	u = pp.u();
+	v = pp.v();
       }
     }
     return SPoint2(u, v);
@@ -247,9 +268,8 @@ int OCCEdge::minimumMeshSegments() const
   else
     np = CTX::instance()->mesh.minCurvPoints - 1;
 
-  // if the edge is closed, ensure that at least 3 points are
-  // generated in the 1D mesh (4 segments, one of which is
-  // degenerated)
+  // if the edge is closed, ensure that at least 3 points are generated in the
+  // 1D mesh (4 segments, one of which is degenerated)
   if (getBeginVertex() == getEndVertex()) np = std::max(4, np);
 
   return std::max(np, meshAttributes.minimumMeshSegments);
@@ -354,11 +374,7 @@ void OCCEdge::replaceEndingPointsInternals(GVertex *g0, GVertex *g1)
     _replacement=E;
   }
   else {
-#if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR < 6)
-    BOPTools_Tools::MakeSplitEdge(aEx, aVR1, t1, aVR2, t2, _replacement);
-#else
     BOPTools_AlgoTools::MakeSplitEdge(aEx, aVR1, t1, aVR2, t2, _replacement);
-#endif
   }
   TopoDS_Edge temp = c;
   c = _replacement;
