@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -6,6 +6,7 @@
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GModel.h"
+#include "GModelIO_OCC.h"
 #include "GEdgeLoop.h"
 #include "OCCVertex.h"
 #include "OCCEdge.h"
@@ -14,44 +15,43 @@
 #include "Context.h"
 
 #if defined(HAVE_OCC)
-#include "GModelIO_OCC.h"
-#include <Standard_Version.hxx>
-#include <Geom_CylindricalSurface.hxx>
-#include <Geom_ConicalSurface.hxx>
-#include <Geom_BSplineSurface.hxx>
-#include <Geom_SphericalSurface.hxx>
-#include <Geom_ToroidalSurface.hxx>
-#include <Geom_SurfaceOfRevolution.hxx>
-#include <Geom_BezierSurface.hxx>
-#include <Geom_Plane.hxx>
-#include <gp_Pln.hxx>
-#include <BRepMesh_FastDiscret.hxx>
 
-#if ((OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6)) || (OCC_VERSION_MAJOR >= 7)
-#  if ((OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR < 8))
-#  include <BOPInt_Context.hxx>
-#  else
-#  include <IntTools_Context.hxx>
-#  endif
-#include <BOPTools_AlgoTools2D.hxx>
 #include <BOPTools_AlgoTools.hxx>
-#else
+#include <BOPTools_AlgoTools2D.hxx>
+#include <BRepBndLib.hxx>
+#include <BRepLProp_SLProps.hxx>
+#include <BRep_Builder.hxx>
+#include <Bnd_Box.hxx>
+#include <GeomAPI_ProjectPointOnSurf.hxx>
+#include <Geom_BSplineSurface.hxx>
+#include <Geom_BezierSurface.hxx>
+#include <Geom_ConicalSurface.hxx>
+#include <Geom_CylindricalSurface.hxx>
+#include <Geom_Plane.hxx>
+#include <Geom_SphericalSurface.hxx>
+#include <Geom_SurfaceOfRevolution.hxx>
+#include <Geom_ToroidalSurface.hxx>
 #include <IntTools_Context.hxx>
-#include <BOPTools_Tools2D.hxx>
-#include <BOPTools_Tools3D.hxx>
-#endif
+#include <ShapeAnalysis.hxx>
+#include <Standard_Version.hxx>
+#include <TopExp_Explorer.hxx>
+#include <TopoDS.hxx>
+#include <gp_Pln.hxx>
 
 OCCFace::OCCFace(GModel *m, TopoDS_Face _s, int num)
   : GFace(m, num), s(_s)
 {
   setup();
-  model()->getOCCInternals()->bind(s, num);
+  if(model()->getOCCInternals())
+    model()->getOCCInternals()->bind(s, num);
 }
 
 OCCFace::~OCCFace()
 {
-  model()->getOCCInternals()->unbind(s);
-  model()->getOCCInternals()->unbind(_replaced);
+  if(model()->getOCCInternals()){
+    model()->getOCCInternals()->unbind(s, tag());
+    model()->getOCCInternals()->unbind(_replaced, tag());
+  }
 }
 
 void OCCFace::setup()
@@ -66,12 +66,14 @@ void OCCFace::setup()
     std::list<GEdge*> l_wire;
     for(exp3.Init(wire, TopAbs_EDGE); exp3.More(); exp3.Next()){
       TopoDS_Edge edge = TopoDS::Edge(exp3.Current());
-      GEdge *e = model()->getOCCInternals()->getOCCEdgeByNativePtr(model(), edge);
+      GEdge *e = 0;
+      if(model()->getOCCInternals())
+        e = model()->getOCCInternals()->getEdgeForOCCShape(model(), edge);
       if(!e){
 	Msg::Error("Unknown edge in face %d", tag());
       }
       else if(edge.Orientation() == TopAbs_INTERNAL){
-        Msg::Info("Adding embedded edge %d", e->tag());
+        Msg::Debug("Adding embedded edge %d in face %d", e->tag(), tag());
         embedded_edges.push_back(e);
         OCCEdge *occe = (OCCEdge*)e;
         occe->setTrimmed(this);
@@ -123,21 +125,16 @@ void OCCFace::setup()
   vmax += fabs(dv) / 100.0;
   occface = BRep_Tool::Surface(s);
 
-  // std::list<GEdge*>::const_iterator it = l_edges.begin();
-  // for (; it != l_edges.end(); ++it){
-  //   printf("edge %d : %d %d iseam %d \n", (*it)->tag(),
-  //          (*it)->getBeginVertex()->tag(), (*it)->getEndVertex()->tag(),
-  //          (*it)->isSeam(this));
-  // }
-
   for(exp2.Init(s, TopAbs_VERTEX); exp2.More(); exp2.Next()){
     TopoDS_Vertex vertex = TopoDS::Vertex(exp2.Current());
-    GVertex *v = model()->getOCCInternals()->getOCCVertexByNativePtr(model(), vertex);
+    GVertex *v = 0;
+    if(model()->getOCCInternals())
+      v = model()->getOCCInternals()->getVertexForOCCShape(model(), vertex);
     if(!v){
       Msg::Error("Unknown vertex in face %d", tag());
     }
     else if(vertex.Orientation() == TopAbs_INTERNAL){
-      Msg::Info("Adding embedded vertex %d", v->tag());
+      Msg::Debug("Adding embedded vertex %d in face %d", v->tag(), tag());
       embedded_vertices.push_back(v);
     }
   }
@@ -146,7 +143,13 @@ void OCCFace::setup()
 SBoundingBox3d OCCFace::bounds() const
 {
   Bnd_Box b;
-  BRepBndLib::Add(s, b);
+  try{
+    BRepBndLib::Add(s, b);
+  }
+  catch(Standard_Failure &err){
+    Msg::Error("OpenCASCADE exception %s", err.GetMessageString());
+    return SBoundingBox3d();
+  }
   double xmin, ymin, zmin, xmax, ymax, zmax;
   b.Get(xmin, ymin, zmin, xmax, ymax, zmax);
   SBoundingBox3d bbox(xmin, ymin, zmin, xmax, ymax, zmax);
@@ -412,34 +415,8 @@ bool OCCFace::buildSTLTriangulation(bool force)
     else
       return true;
   }
-
-  Bnd_Box aBox;
-  BRepBndLib::Add(s, aBox);
-
-#if (OCC_VERSION_MAJOR >= 7)
-  BRepMesh_FastDiscret::Parameters parameters;
-  parameters.Deflection = 0.1;
-  parameters.Angle = 0.5;
-  parameters.Relative = Standard_True;
-  BRepMesh_FastDiscret aMesher(aBox, parameters);
-#else
-  BRepMesh_FastDiscret aMesher(0.1, 0.5, aBox, Standard_False, Standard_False,
-                               Standard_True, Standard_False);
-#endif
-#if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR < 5)
-  aMesher.Add(s);
-#else
-  aMesher.Perform(s);
-#endif
-
-  TopLoc_Location loc;
-  Handle(Poly_Triangulation) triangulation = BRep_Tool::Triangulation(s, loc);
-
-  if(triangulation.IsNull() || !triangulation->HasUVNodes()){
-    if(triangulation.IsNull())
-      Msg::Warning("OCC STL triangulation of surface %d failed", tag());
-    else
-      Msg::Warning("OCC STL triangulation of surface %d has no u,v coordinates", tag());
+  if(!model()->getOCCInternals()->makeFaceSTL(s, stl_vertices, stl_triangles)){
+    Msg::Warning("OpenCASCADE triangulation of surface %d failed", tag());
     // add a dummy triangle so that we won't try again
     stl_vertices.push_back(SPoint2(0., 0.));
     stl_triangles.push_back(0);
@@ -448,29 +425,18 @@ bool OCCFace::buildSTLTriangulation(bool force)
     return false;
   }
 
-  for(int i = 1; i <= triangulation->NbNodes(); i++){
-    gp_Pnt2d p = (triangulation->UVNodes())(i);
-    stl_vertices.push_back(SPoint2(p.X(), p.Y()));
-  }
-
   bool reverse = false;
-  for(int i = 1; i <= triangulation->NbTriangles(); i++){
-    Poly_Triangle triangle = (triangulation->Triangles())(i);
-    int p1, p2, p3;
-    triangle.Get(p1, p2, p3);
-
-    // orient STL mesh to get normal right
-    if(i == 1){
-      gp_Pnt2d gp1 = (triangulation->UVNodes())(p1);
-      gp_Pnt2d gp2 = (triangulation->UVNodes())(p2);
-      gp_Pnt2d gp3 = (triangulation->UVNodes())(p3);
-      SPoint2 b = SPoint2(gp1.X(), gp1.Y()) + SPoint2(gp2.X(), gp2.Y()) +
-        SPoint2(gp3.X(), gp3.Y());
+  for(unsigned int i = 0; i < stl_triangles.size(); i += 3){
+    if(i == 0){
+      SPoint2 gp1 = stl_vertices[stl_triangles[i]];
+      SPoint2 gp2 = stl_vertices[stl_triangles[i + 1]];
+      SPoint2 gp3 = stl_vertices[stl_triangles[i + 2]];
+      SPoint2 b = gp1 + gp2 + gp2;
       b *= 1. / 3.;
       SVector3 nf = normal(b);
-      GPoint sp1 = point(gp1.X(), gp1.Y());
-      GPoint sp2 = point(gp2.X(), gp2.Y());
-      GPoint sp3 = point(gp3.X(), gp3.Y());
+      GPoint sp1 = point(gp1.x(), gp1.y());
+      GPoint sp2 = point(gp2.x(), gp2.y());
+      GPoint sp3 = point(gp3.x(), gp3.y());
       double n[3];
       normal3points(sp1.x(), sp1.y(), sp1.z(),
                     sp2.x(), sp2.y(), sp2.z(),
@@ -481,34 +447,19 @@ bool OCCFace::buildSTLTriangulation(bool force)
         reverse = true;
       }
     }
-
-    if(!reverse){
-      stl_triangles.push_back(p1 - 1);
-      stl_triangles.push_back(p2 - 1);
-      stl_triangles.push_back(p3 - 1);
-    }
-    else{
-      stl_triangles.push_back(p1 - 1);
-      stl_triangles.push_back(p3 - 1);
-      stl_triangles.push_back(p2 - 1);
+    if(reverse){
+      int tmp = stl_triangles[i + 1];
+      stl_triangles[i + 1] = stl_triangles[i + 2];
+      stl_triangles[i + 2] = tmp;
     }
   }
-
   return true;
 }
 
 void OCCFace::replaceEdgesInternal(std::list<GEdge*> &new_edges)
 {
-
-#if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6) && (OCC_VERSION_MINOR < 8)
-  Handle(BOPInt_Context) myContext = new BOPInt_Context;
-#elif defined(OCC_VERSION_HEX) && OCC_VERSION_HEX >= 0x060503
-  Handle(IntTools_Context) myContext = new IntTools_Context;
-#else
-  IntTools_Context myContext;
-#endif
-
   // we simply replace old edges by new edges in the structure
+  Handle(IntTools_Context) myContext = new IntTools_Context;
 
   // make a copy of s
   TopoDS_Face copy_of_s_forward = s;
@@ -566,11 +517,7 @@ void OCCFace::replaceEdgesInternal(std::list<GEdge*> &new_edges)
 	      continue;
 	    }
 	    else{
-#if ((OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6)) || (OCC_VERSION_MAJOR >= 7)
 	      aTx = BOPTools_AlgoTools2D::IntermediatePoint(aT1, aT2);
-#else
-	      aTx = BOPTools_Tools2D::IntermediatePoint(aT1, aT2);
-#endif
 	      gp_Pnt2d aP2D;
 	      aC2D->D0(aTx, aP2D);
 	      aUx=aP2D.X();
@@ -582,18 +529,10 @@ void OCCFace::replaceEdgesInternal(std::list<GEdge*> &new_edges)
 	    }
 	  }
 	}
-#if ((OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6)) || (OCC_VERSION_MAJOR >= 7)
 	BOPTools_AlgoTools2D::BuildPCurveForEdgeOnFace(aER, copy_of_s_forward);
-#else
-	BOPTools_Tools2D::BuildPCurveForEdgeOnFace(aER, copy_of_s_forward);
-#endif
 	// orient image
 	Standard_Boolean bIsToReverse =
-#if ((OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6)) || (OCC_VERSION_MAJOR >= 7)
           BOPTools_AlgoTools::IsSplitToReverse(aER, aE, myContext);
-#else
-          BOPTools_Tools3D::IsSplitToReverse1(aER, aE, myContext);
-#endif
 	if (bIsToReverse) {
 	  aER.Reverse();
 	}
@@ -609,7 +548,8 @@ void OCCFace::replaceEdgesInternal(std::list<GEdge*> &new_edges)
   s = newFace;
 
   setup();
-  model()->getOCCInternals()->bind(_replaced, tag());
+  if(model()->getOCCInternals())
+    model()->getOCCInternals()->bind(_replaced, tag());
 }
 
 bool OCCFace::isSphere (double &radius, SPoint3 &center) const
