@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -9,12 +9,12 @@
 #include "GmshConfig.h"
 #include "GFace.h"
 #include "GModel.h"
-#include "OCCVertex.h"
-#include "OCCEdge.h"
 #include "Range.h"
 
 #if defined(HAVE_OCC)
 
+#include <TopoDS_Face.hxx>
+
 class OCCFace : public GFace {
  protected:
   TopoDS_Face _replaced;
