@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <stdlib.h>
 #include "GModel.h"
@@ -21,19 +21,44 @@
 #endif
 
 gmshFace::gmshFace(GModel *m, Surface *face)
-  : GFace(m, face->Num), s(face), isSphere(false), radius(0.)
+  : GFace(m, face->Num), isSphere(false), radius(0.)
 {
+  resetNativePtr(face);
   resetMeshAttributes();
+}
 
-  // edgeCounterparts = s->edgeCounterparts;
-  // affineTransform = s->affineTransform;
+// a face is degenerate if 
+bool gmshFace::degenerate(int dim) const {
+  std::list<GEdge*> eds = edges();
+  int numNonDegenerate = 0;
+  std::set<GEdge*> t;
+  for(std::list<GEdge*>::iterator it = eds.begin(); it != eds.end(); ++it){
+    GEdge *e = *it;
+    GVertex *start = e->getBeginVertex();
+    GVertex *next  = e->getEndVertex();
+    if (start != next && t.find(e) == t.end()){
+      numNonDegenerate++;
+    }
+    t.insert(e);
+  }
+  //  printf("%d \n",numNonDegenerate);
+  return numNonDegenerate <= 1;
+}
+
+
+void gmshFace::resetNativePtr(Surface *face)
+{
+  s = face;
+  l_edges.clear();
+  l_dirs.clear();
+  edgeLoops.clear();
 
   std::vector<GEdge*> eds;
   std::vector<int> nums;
   for(int i = 0; i < List_Nbr(s->Generatrices); i++){
     Curve *c;
     List_Read(s->Generatrices, i, &c);
-    GEdge *e = m->getEdgeByTag(abs(c->Num));
+    GEdge *e = model()->getEdgeByTag(abs(c->Num));
     if(e){
       eds.push_back(e);
       nums.push_back(c->Num);
@@ -44,7 +69,7 @@ gmshFace::gmshFace(GModel *m, Surface *face)
   for(int i = 0; i < List_Nbr(s->GeneratricesByTag); i++){
     int j;
     List_Read(s->GeneratricesByTag, i, &j);
-    GEdge *e = m->getEdgeByTag(abs(j));
+    GEdge *e = model()->getEdgeByTag(abs(j));
     if(e){
       eds.push_back(e);
       nums.push_back(j);
@@ -80,29 +105,7 @@ gmshFace::gmshFace(GModel *m, Surface *face)
   // the bounding vertices)
   if(s->Typ == MSH_SURF_PLAN) computeMeanPlane();
 
-  if(s->EmbeddedCurves){
-    for(int i = 0; i < List_Nbr(s->EmbeddedCurves); i++){
-      Curve *c;
-      List_Read(s->EmbeddedCurves, i, &c);
-      GEdge *e = m->getEdgeByTag(abs(c->Num));
-      if(e)
-        addEmbeddedEdge(e);
-      else
-        Msg::Error("Unknown curve %d", c->Num);
-    }
-  }
-  if(s->EmbeddedPoints){
-    for(int i = 0; i < List_Nbr(s->EmbeddedPoints); i++){
-      Vertex *v;
-      List_Read(s->EmbeddedPoints, i, &v);
-      GVertex *gv = m->getVertexByTag(v->Num);
-      if(gv)
-        embedded_vertices.push_back(gv);
-      else
-        Msg::Error("Unknown point %d", v->Num);
-    }
-  }
-  isSphere = iSRuledSurfaceASphere(s, center, radius);
+  isSphere = IsRuledSurfaceASphere(s, center, radius);
 }
 
 double gmshFace::getMetricEigenvalue(const SPoint2 &pt)
