@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -19,15 +19,16 @@
 #include "GmshConfig.h"
 #include "Context.h"
 #include "Field.h"
-#include "GeoInterpolation.h"
 #include "GModel.h"
+#include "GModelIO_GEO.h"
 #include "GmshMessage.h"
 #include "Numeric.h"
 #include "mathEvaluator.h"
 #include "BackgroundMeshTools.h"
-#include "CenterlineField.h"
 #include "STensor3.h"
 #include "meshMetric.h"
+#include "ExtrudeParams.h"
+
 #if defined(HAVE_POST)
 #include "PView.h"
 #include "OctreePost.h"
@@ -1498,6 +1499,7 @@ class PostViewField : public Field
   }
   double operator() (double x, double y, double z, GEntity *ge=0)
   {
+
     PView *v = getView();
     if(!v) return MAX_LC;
     if(update_needed){
@@ -1527,7 +1529,7 @@ class PostViewField : public Field
     // of finding an element
     if(!octree->searchTensorWithTol(x, y, z, l, 0, 0, 0.05))
       Msg::Info("No tensor element found containing point (%g,%g,%g)", x, y, z);
-    if(crop_negative_values){
+    if(0 && crop_negative_values){
       if(l[0] <= 0 && l[1] <= 0 && l[2] <= 0 &&
          l[3] <= 0 && l[4] <= 0 && l[5] <= 0 &&
          l[6] <= 0 && l[7] <= 0 && l[8] <= 0){
@@ -1539,6 +1541,7 @@ class PostViewField : public Field
         }
       }
     }
+    //    printf("%g %g %g %g %g %g %g %g %g\n",l[0],l[1], l[2],l[3],l[4], l[5],l[6],l[7], l[8]);
     metr(0, 0) = l[0]; metr(0, 1) = l[1]; metr(0, 2) = l[2];
     metr(1, 0) = l[3]; metr(1, 1) = l[4]; metr(1, 2) = l[5];
     metr(2, 0) = l[6]; metr(2, 1) = l[7]; metr(2, 2) = l[8];
@@ -1858,6 +1861,8 @@ class AttractorAnisoCurveField : public Field {
       (lMinNormal, "Minimum mesh size in the direction normal to the closest curve.");
     options["lMaxNormal"] = new FieldOptionDouble
       (lMaxNormal, "Maximum mesh size in the direction normal to the closest curve.");
+    // make sure all internal GEO CAD data has been synced with GModel
+    GModel::current()->getGEOInternals()->synchronize(GModel::current());
   }
   virtual bool isotropic () const {return false;}
   ~AttractorAnisoCurveField()
@@ -1893,38 +1898,22 @@ class AttractorAnisoCurveField : public Field {
     int k = 0;
     for(std::list<int>::iterator it = edges_id.begin();
         it != edges_id.end(); ++it) {
-      Curve *c = FindCurve(*it);
-      if(c) {
+      GEdge *e = GModel::current()->getEdgeByTag(*it);
+      if(e) {
         for(int i = 1; i < n_nodes_by_edge - 1; i++) {
           double u = (double)i / (n_nodes_by_edge - 1);
-          Vertex V = InterpolateCurve(c, u, 0);
-          zeronodes[k][0] = V.Pos.X;
-          zeronodes[k][1] = V.Pos.Y;
-          zeronodes[k][2] = V.Pos.Z;
-          Vertex V2 = InterpolateCurve(c, u, 1);
-          tg[k] = SVector3(V2.Pos.X, V2.Pos.Y, V2.Pos.Z);
+          Range<double> b = e->parBounds(0);
+          double t = b.low() + u * (b.high() - b.low());
+          GPoint gp = e->point(t);
+          SVector3 d = e->firstDer(t);
+          zeronodes[k][0] = gp.x();
+          zeronodes[k][1] = gp.y();
+          zeronodes[k][2] = gp.z();
+          tg[k] = d;
           tg[k].normalize();
           k++;
         }
       }
-      else {
-        GEdge *e = GModel::current()->getEdgeByTag(*it);
-        if(e) {
-          for(int i = 1; i < n_nodes_by_edge - 1; i++) {
-            double u = (double)i / (n_nodes_by_edge - 1);
-            Range<double> b = e->parBounds(0);
-            double t = b.low() + u * (b.high() - b.low());
-            GPoint gp = e->point(t);
-            SVector3 d = e->firstDer(t);
-            zeronodes[k][0] = gp.x();
-            zeronodes[k][1] = gp.y();
-            zeronodes[k][2] = gp.z();
-            tg[k] = d;
-            tg[k].normalize();
-            k++;
-          }
-        }
-      }
     }
     kdtree = new ANNkd_tree(zeronodes, totpoints, 3);
     update_needed = false;
@@ -1934,6 +1923,9 @@ class AttractorAnisoCurveField : public Field {
     if(update_needed)
       update();
     double xyz[3] = { x, y, z };
+#if defined(_OPENMP)
+#pragma omp critical
+#endif
     kdtree->annkSearch(xyz, 1, index, dist);
     double d = sqrt(dist[0]);
     double lTg = d < dMin ? lMinTangent : d > dMax ? lMaxTangent :
@@ -1951,6 +1943,9 @@ class AttractorAnisoCurveField : public Field {
     if(update_needed)
       update();
     double xyz[3] = { X, Y, Z };
+#if defined(_OPENMP)
+#pragma omp critical
+#endif
     kdtree->annkSearch(xyz, 1, index, dist);
     double d = sqrt(dist[0]);
     return std::max(d, 0.05);
@@ -1961,14 +1956,14 @@ class AttractorField : public Field
 {
   ANNkd_tree *kdtree;
   ANNpointArray zeronodes;
-  ANNidxArray index;
-  ANNdistArray dist;
   std::list<int> nodes_id, edges_id, faces_id;
   std::vector<AttractorInfo> _infos;
   int _xFieldId, _yFieldId, _zFieldId;
   Field *_xField, *_yField, *_zField;
   int n_nodes_by_edge;
- public:
+  ANNidxArray index;
+  ANNdistArray dist;
+public:
   AttractorField(int dim, int tag, int nbe)
     : kdtree(0), zeronodes(0), n_nodes_by_edge(nbe)
   {
@@ -2007,10 +2002,10 @@ class AttractorField : public Field
   }
   ~AttractorField()
   {
-    if(kdtree) delete kdtree;
-    if(zeronodes) annDeallocPts(zeronodes);
     delete[]index;
     delete[]dist;
+    if(kdtree) delete kdtree;
+    if(zeronodes) annDeallocPts(zeronodes);
   }
   const char *getName()
   {
@@ -2033,16 +2028,16 @@ class AttractorField : public Field
   std::pair<AttractorInfo,SPoint3> getAttractorInfo() const
   {
     return std::make_pair(_infos[index[0]], SPoint3(zeronodes[index[0]][0],
-                                                    zeronodes[index[0]][1],
-                                                    zeronodes[index[0]][2]));
+						    zeronodes[index[0]][1],
+						    zeronodes[index[0]][2]));
   }
-  virtual double operator() (double X, double Y, double Z, GEntity *ge=0)
-  {
-    _xField = _xFieldId >= 0 ? (GModel::current()->getFields()->get(_xFieldId)) : NULL;
-    _yField = _yFieldId >= 0 ? (GModel::current()->getFields()->get(_yFieldId)) : NULL;
-    _zField = _zFieldId >= 0 ? (GModel::current()->getFields()->get(_zFieldId)) : NULL;
 
+  void update() {
     if(update_needed) {
+      //      printf("updating\n");
+      _xField = _xFieldId >= 0 ? (GModel::current()->getFields()->get(_xFieldId)) : NULL;
+      _yField = _yFieldId >= 0 ? (GModel::current()->getFields()->get(_yFieldId)) : NULL;
+      _zField = _zFieldId >= 0 ? (GModel::current()->getFields()->get(_zFieldId)) : NULL;
       if(zeronodes) {
         annDeallocPts(zeronodes);
         delete kdtree;
@@ -2178,11 +2173,359 @@ class AttractorField : public Field
       kdtree = new ANNkd_tree(zeronodes, totpoints, 3);
       update_needed = false;
     }
+  }
 
+  virtual double operator() (double X, double Y, double Z, GEntity *ge=0)
+  {
+#if defined(_OPENMP)
+#pragma omp critical
+#endif
+    {
+      update();
+    }
+    // NOT thread safe anyway :-)
     double xyz[3];
     getCoord(X, Y, Z, xyz[0], xyz[1], xyz[2], ge);
     kdtree->annkSearch(xyz, 1, index, dist);
-    return sqrt(dist[0]);
+    double d = dist[0];
+    return sqrt(d);
+  }
+};
+
+//------------------------ N A N O F L A N N -------------------------------------------------------------------------
+#include <nanoflann.hpp>
+using namespace nanoflann;
+// This is an exampleof a custom data set class
+struct PointCloud
+{
+  std::vector<SPoint3>  pts;
+}; 
+
+// And this is the "dataset to kd-tree" adaptor class:
+template <typename Derived>
+struct PointCloudAdaptor
+{
+  
+  const Derived &obj; //!< A const ref to the data set origin
+  
+  /// The constructor that sets the data set source
+  PointCloudAdaptor(const Derived &obj_) : obj(obj_) { }
+  
+  /// CRTP helper method
+  inline const Derived& derived() const { return obj; }
+  
+  // Must return the number of data points
+  inline size_t kdtree_get_point_count() const { return derived().pts.size(); }
+  
+  // Returns the distance between the vector "p1[0:size-1]" and the data point with index "idx_p2" stored in the class:
+  inline double kdtree_distance(const double *p1, const size_t idx_p2,size_t /*size*/) const
+  {
+    const double d0=p1[0]-derived().pts[idx_p2].x();
+    const double d1=p1[1]-derived().pts[idx_p2].y();
+    const double d2=p1[2]-derived().pts[idx_p2].z();
+    return d0*d0+d1*d1+d2*d2;
+  }
+
+  // Returns the dim'th component of the idx'th point in the class:
+  // Since this is inlined and the "dim" argument is typically an immediate value, the
+  //  "if/else's" are actually solved at compile time.
+  inline double kdtree_get_pt(const size_t idx, int dim) const
+  {
+    if (dim==0) return derived().pts[idx].x();
+    else if (dim==1) return derived().pts[idx].y();
+    else return derived().pts[idx].z();
+  }
+  
+  // Optional bounding-box computation: return false to default to a standard bbox computation loop.
+  //   Return true if the BBOX was already computed by the class and returned in "bb" so it can be avoided to redo it again.
+  //   Look at bb.size() to find out the expected dimensionality (e.g. 2 or 3 for point clouds)
+  template <class BBOX>
+  bool kdtree_get_bbox(BBOX& /*bb*/) const { return false; }
+
+}; // end of PointCloudAdaptor
+
+
+typedef PointCloudAdaptor<PointCloud> PC2KD;
+
+typedef KDTreeSingleIndexAdaptor<
+  L2_Simple_Adaptor<double, PC2KD > ,
+  PC2KD, 3 > my_kd_tree_t;
+
+class DistanceField : public Field
+{  
+  std::list<int> nodes_id, edges_id, faces_id;
+  int _xFieldId, _yFieldId, _zFieldId;
+  Field *_xField, *_yField, *_zField;
+  int n_nodes_by_edge;
+  PointCloud P;
+  my_kd_tree_t *index;
+  PC2KD pc2kd;
+public:
+  DistanceField() : index(NULL) , pc2kd(P)
+  {
+    n_nodes_by_edge = 20;
+    options["NodesList"] = new FieldOptionList
+      (nodes_id, "Indices of nodes in the geometric model", &update_needed);
+    options["EdgesList"] = new FieldOptionList
+      (edges_id, "Indices of curves in the geometric model", &update_needed);
+    options["NNodesByEdge"] = new FieldOptionInt
+      (n_nodes_by_edge, "Number of nodes used to discretized each curve",
+       &update_needed);
+    options["FacesList"] = new FieldOptionList
+      (faces_id, "Indices of surfaces in the geometric model (Warning, this feature "
+       "is still experimental. It might (read: will probably) give wrong results "
+       "for complex surfaces)", &update_needed);
+    _xFieldId = _yFieldId = _zFieldId = -1;
+    options["FieldX"] = new FieldOptionInt
+      (_xFieldId, "Id of the field to use as x coordinate.", &update_needed);
+    options["FieldY"] = new FieldOptionInt
+      (_yFieldId, "Id of the field to use as y coordinate.", &update_needed);
+    options["FieldZ"] = new FieldOptionInt
+      (_zFieldId, "Id of the field to use as z coordinate.", &update_needed);
+  }
+  ~DistanceField()
+  {
+    if (index) delete index;
+  }
+  const char *getName()
+  {
+    return "DistanceField";
+  }
+  std::string getDescription()
+  {
+    return "Compute the distance from the nearest node in a list. It can also "
+      "be used to compute the distance from curves, in which case each curve "
+      "is replaced by NNodesByEdge equidistant nodes and the distance from those "
+      "nodes is computed.";
+  }
+  void update() {
+    if(update_needed) {
+      //      printf("Creating Data For Distance Field\n");
+      _xField = _xFieldId >= 0 ? (GModel::current()->getFields()->get(_xFieldId)) : NULL;
+      _yField = _yFieldId >= 0 ? (GModel::current()->getFields()->get(_yFieldId)) : NULL;
+      _zField = _zFieldId >= 0 ? (GModel::current()->getFields()->get(_zFieldId)) : NULL;
+
+      std::vector<SPoint3> &points = P.pts;
+      for(std::list<int>::iterator it = faces_id.begin();
+          it != faces_id.end(); ++it) {
+	GFace *f = GModel::current()->getFaceByTag(*it);
+	if (f){
+	  if (f->mesh_vertices.size()){
+	    for (unsigned int i = 0; i < f->mesh_vertices.size(); i++){
+	      MVertex *v = f->mesh_vertices[i];
+	      points.push_back(SPoint3(v->x(), v->y(), v->z()));
+	    }
+	  }
+	  else {
+	    SBoundingBox3d bb = f->bounds();
+	    SVector3 dd = bb.max() - bb.min();
+	    double maxDist = dd.norm() / n_nodes_by_edge ;
+	    std::vector<SPoint2> uvpoints;
+	    f->fillPointCloud(maxDist, &points, &uvpoints);
+	  }
+	}
+      }
+      
+      for(std::list<int>::iterator it = nodes_id.begin();
+          it != nodes_id.end(); ++it) {
+	GVertex *gv = GModel::current()->getVertexByTag(*it);
+	if(gv) points.push_back(SPoint3(gv->x(), gv->y(), gv->z()));
+      }
+
+      for(std::list<int>::iterator it = edges_id.begin();
+          it != edges_id.end(); ++it) {
+	GEdge *e = GModel::current()->getEdgeByTag(*it);
+	if(e) {
+	  if (e->mesh_vertices.size()){
+	    for(unsigned int i = 0; i < e->mesh_vertices.size(); i++) 
+              points.push_back(SPoint3(e->mesh_vertices[i]->x(),
+				       e->mesh_vertices[i]->y(),
+				       e->mesh_vertices[i]->z()));
+	  }
+	  int NNN = n_nodes_by_edge - e->mesh_vertices.size();
+	  for(int i = 1; i < NNN - 1; i++) {
+	    double u = (double)i / (NNN - 1);
+	    Range<double> b = e->parBounds(0);
+	    double t = b.low() + u * (b.high() - b.low());
+	    GPoint gp = e->point(t);
+	    points.push_back(SPoint3(gp.x(),gp.y(),gp.z()));
+	  }
+	}
+      }      
+      
+      // construct a kd-tree index:
+      
+      //      printf("Constructing kd-tree with %lu points\n",points.size());
+      
+      index = new my_kd_tree_t(3 , pc2kd, KDTreeSingleIndexAdaptorParams(10) );
+      index->buildIndex();
+      update_needed=false;
+    }
+  }
+  
+  virtual double operator() (double X, double Y, double Z, GEntity *ge=0)
+  {
+    double query_pt[3] = {X,Y,Z};
+    const size_t num_results = 1;
+    size_t ret_index;
+    double out_dist_sqr;
+    nanoflann::KNNResultSet<double> resultSet(num_results);
+    resultSet.init(&ret_index, &out_dist_sqr );
+    index->findNeighbors(resultSet, &query_pt[0], nanoflann::SearchParams(10));
+    return sqrt (out_dist_sqr);
+  }
+};
+//------------------------ N A N O F L A N N -------------------------------------------------------------------------
+
+
+class OctreeField : public Field {
+  // octree field
+  class Cell {
+    void *_data;
+    bool _isleaf;
+    public:
+    double evaluate(double x, double y, double z) const {
+      if (_isleaf)
+        return *(double*)_data;
+      Cell *sub = (Cell*)_data;
+      int i = x > 0.5 ? 1 : 0;
+      int j = y > 0.5 ? 1 : 0;
+      int k = z > 0.5 ? 1 : 0;
+      return sub[i*4+j*2+k].evaluate(2*x-i, 2*y-j, 2*z-k);
+    }
+    Cell(){};
+    void init(double x0, double y0, double z0, double l, Field &field, int level) {
+#if 0
+      double v[8] = {
+        field(x0,y0,z0),field(x0,y0,z0+l),field(x0,y0+l,z0),field(x0,y0+l,z0+l),
+        field(x0+l,y0,z0),field(x0+l,y0,z0+l),field(x0+l,y0+l,z0),field(x0+l,y0+l,z0+l)
+      };
+      double dmax = 0;
+      double vmin = v[0];
+      double vc = field(x0+l/2,y0+l/2,z0+l/2);
+      for (int i = 0; i < 8; ++i){
+        dmax = std::max(dmax, std::abs(vc-v[i]));
+        vmin = std::min(vmin, v[i]);
+      }
+#else
+      double dmax = 0;
+      double vc = field(x0+l/2,y0+l/2,z0+l/2);
+      double vmin = vc;
+      bool split = level > 0;
+      if (level > -4) {
+#define NSAMPLE 2
+        double dl = l/NSAMPLE;
+        for (int i = 0; i <= NSAMPLE; ++i){
+          for (int j = 0; j <= NSAMPLE; ++j){
+            for (int k = 0; k <= NSAMPLE; ++k){
+              double w = field(x0 + i*dl, y0+j*dl, z0+k*dl);
+              dmax = std::max(dmax, std::abs(vc-w));
+              vmin = std::min(vmin, w);
+              split |= (dmax/vmin > 0.2 && vmin < l);
+              if(split)
+                break;
+            }
+          }
+        }
+#endif
+      }
+      if (split) {
+        _isleaf = false;
+        Cell *sub = new Cell[8];
+        double l2 = l/2;
+        sub[0].init(x0, y0, z0, l2, field, level-1);
+        sub[1].init(x0, y0, z0+l2, l2, field, level-1);
+        sub[2].init(x0, y0+l2, z0, l2, field, level-1);
+        sub[3].init(x0, y0+l2, z0+l2, l2, field, level-1);
+        sub[4].init(x0+l2, y0, z0, l2, field, level-1);
+        sub[5].init(x0+l2, y0, z0+l2, l2, field, level-1);
+        sub[6].init(x0+l2, y0+l2, z0, l2, field, level-1);
+        sub[7].init(x0+l2, y0+l2, z0+l2, l2, field, level-1);
+        _data = (void*)sub;
+      }
+      else {
+        _isleaf = true;
+        _data = (void*)new double;
+        *(double*)_data = vc;
+      }
+    }
+    ~Cell() {
+      if (_isleaf) {
+        delete (double*)_data;
+      }
+      else {
+        Cell *sub = (Cell*)_data;
+        delete []sub;
+      }
+    }
+    void print(double x0, double y0, double z0, double l, FILE *f) {
+      if (_isleaf) {
+        fprintf(f, "SP(%g, %g, %g) {%g};\n", x0+l/2, y0+l/2, z0+l/2, *(double*)_data);
+      }
+      else {
+        Cell *sub = (Cell*)_data;
+        double l2 = l/2;
+        sub[0].print(x0, y0, z0, l2, f);
+        sub[1].print(x0, y0, z0+l2, l2, f);
+        sub[2].print(x0, y0+l2, z0, l2, f);
+        sub[3].print(x0, y0+l2, z0+l2, l2, f);
+        sub[4].print(x0+l2, y0, z0, l2, f);
+        sub[5].print(x0+l2, y0, z0+l2, l2, f);
+        sub[6].print(x0+l2, y0+l2, z0, l2, f);
+        sub[7].print(x0+l2, y0+l2, z0+l2, l2, f);
+      }
+    }
+  };
+  Cell *_root;
+  int _inFieldId;
+  Field *_inField;
+  SBoundingBox3d bounds;
+  double _l0;
+ public:
+  OctreeField()
+  {
+    options["InField"] = new FieldOptionInt
+      (_inFieldId, "Id of the field to use as x coordinate.", &update_needed);
+    _root = NULL;
+  }
+  ~OctreeField()
+  {
+    if (_root)
+      delete _root;
+  }
+  const char *getName()
+  {
+    return "Octree";
+  }
+  std::string getDescription()
+  {
+    return "Pre compute another field on an octree to speed-up evalution";
+  }
+  void update(){
+    if(update_needed) {
+      update_needed = false;
+      if (_root) {
+        delete _root;
+        _root = NULL;
+      }
+    }
+    if(!_root) {
+      _inField = _inFieldId >= 0 ? (GModel::current()->getFields()->get(_inFieldId)) : NULL;
+      if (!_inField)
+        return;
+      GModel::current()->getFields()->get(_inFieldId)->update();
+      bounds = GModel::current()->bounds();
+      _root = new Cell;
+      SVector3 d = bounds.max() - bounds.min();
+      _l0 = std::max(std::max(d.x(), d.y()), d.z());
+      _root->init(bounds.min().x(), bounds.min().y(), bounds.min().z(), _l0, *_inField, 4);
+    }
+  }
+  virtual double operator() (double X, double Y, double Z, GEntity *ge=0)
+  {
+    SPoint3 xmin = bounds.min();
+    SVector3 d = bounds.max() - xmin;
+    return _root->evaluate((X-xmin.x())/_l0,(Y-xmin.y())/_l0,(Z-xmin.z())/_l0);
   }
 };
 
@@ -2542,7 +2885,6 @@ FieldManager::FieldManager()
   map_type_name["Threshold"] = new FieldFactoryT<ThresholdField>();
 #if defined(HAVE_ANN)
   map_type_name["BoundaryLayer"] = new FieldFactoryT<BoundaryLayerField>();
-  map_type_name["Centerline"] = new FieldFactoryT<Centerline>();
 #endif
   map_type_name["Box"] = new FieldFactoryT<BoxField>();
   map_type_name["Cylinder"] = new FieldFactoryT<CylinderField>();
@@ -2553,6 +2895,10 @@ FieldManager::FieldManager()
   map_type_name["PostView"] = new FieldFactoryT<PostViewField>();
 #endif
   map_type_name["Gradient"] = new FieldFactoryT<GradientField>();
+#if defined(HAVE_ANN)
+  map_type_name["Octree"] = new FieldFactoryT<OctreeField>();
+#endif
+  map_type_name["Distance"] = new FieldFactoryT<DistanceField>();
   map_type_name["Restrict"] = new FieldFactoryT<RestrictField>();
   map_type_name["Min"] = new FieldFactoryT<MinField>();
   map_type_name["MinAniso"] = new FieldFactoryT<MinAnisoField>();
@@ -2574,6 +2920,12 @@ FieldManager::FieldManager()
   _boundaryLayer_field = -1;
 }
 
+void FieldManager::initialize(){
+  std::map<int,Field*> :: iterator it = begin();
+  for (; it != end() ; ++it) it->second->update();
+}
+
+
 FieldManager::~FieldManager()
 {
   for(std::map<std::string, FieldFactory*>::iterator it = map_type_name.begin();
