@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -29,16 +29,18 @@
 #include "HighOrder.h"
 #include "Generator.h"
 #include "meshGFaceLloyd.h"
-#include "CenterlineField.h"
 #include "GFaceCompound.h"
 #include "Field.h"
 #include "Options.h"
 #include "simple3D.h"
 #include "yamakawa.h"
 #include "meshGRegionRelocateVertex.h"
-
 #include "pointInsertion.h"
 
+#if defined(_OPENMP)
+#include <omp.h>
+#endif
+
 #if defined(HAVE_OPTHOM)
 #include "OptHomRun.h"
 #include "OptHomElastic.h"
@@ -49,11 +51,79 @@
 #include "PViewData.h"
 #endif
 
+class TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES {
+public:
+  void operator () (GRegion *gr) {
+    std::list<GEdge*> e = gr->embeddedEdges();
+    std::list<GFace*> f = gr->embeddedFaces();
+    if (e.empty() && f.empty())return;
+    std::map<MEdge,GEdge*,Less_Edge> edges;
+    std::map<MFace,GFace*,Less_Face> faces;
+    std::list<GEdge*>::iterator it = e.begin();
+    std::list<GFace*>::iterator itf = f.begin();
+    for ( ; it != e.end() ; ++it){
+      for (unsigned int i=0;i<(*it)->lines.size(); ++i){
+	if (distance ((*it)->lines[i]->getVertex(0),(*it)->lines[i]->getVertex(1)) > 1.e-12)
+	  edges.insert(std::make_pair(MEdge((*it)->lines[i]->getVertex(0),(*it)->lines[i]->getVertex(1)),*it));
+      }
+    }
+    for ( ; itf != f.end() ; ++itf){
+      for (unsigned int i=0;i<(*itf)->triangles.size(); ++i){
+	faces.insert(std::make_pair(MFace((*itf)->triangles[i]->getVertex(0),(*itf)->triangles[i]->getVertex(1),(*itf)->triangles[i]->getVertex(2)),*itf));
+      }
+    }
+    Msg::Info ("Searching for %d embedded mesh edges and %d embedded mesh faces in region %d", edges.size(),  faces.size(), gr->tag());
+    for (unsigned int k=0;k<gr->getNumMeshElements();k++){
+      for (int j=0;j<gr->getMeshElement(k)->getNumEdges();j++){
+	edges.erase (gr->getMeshElement(k)->getEdge(j));
+      }
+      for (int j=0;j<gr->getMeshElement(k)->getNumFaces();j++){
+	faces.erase (gr->getMeshElement(k)->getFace(j));
+      }
+    }
+    if (edges.empty() && faces.empty()) {
+      Msg::Info ("All embedded edges and faces are present in the final mesh");
+    }
+    if (edges.size()) {
+      char name[256];
+      sprintf(name,"missingEdgesOnRegion%d.pos",gr->tag());
+      Msg::Error("Region %d : %d mesh edges that should be embedded are missing in the final mesh",gr->tag(), (int)edges.size());
+      Msg::Error("Saving the missing edges in file %s",name);
+      FILE *f = fopen(name,"w");
+      fprintf(f,"View \" \" {\n");
+      for (std::map<MEdge,GEdge*,Less_Edge>::iterator it =  edges.begin() ; it != edges.end(); ++it){
+	MVertex *v1 = it->first.getVertex(0);
+	MVertex *v2 = it->first.getVertex(1);
+	fprintf(f,"SL(%g,%g,%g,%g,%g,%g){%d,%d};\n",v1->x(),v1->y(),v1->z(),v2->x(),v2->y(),v2->z(), it->second->tag(),it->second->tag());
+      }
+      fprintf(f,"};\n");
+      fclose(f);
+    }
+    if (faces.size()) {
+      char name[256];
+      sprintf(name,"missingFacesOnRegion%d.pos",gr->tag());
+      Msg::Error("Region %d : %d mesh faces that should be embedded are missing in the final mesh",gr->tag(), (int)faces.size());
+      Msg::Error("Saving the missing faces in file %s",name);
+      FILE *f = fopen(name,"w");
+      fprintf(f,"View \" \" {\n");
+      for (std::map<MFace,GFace*,Less_Face>::iterator it =  faces.begin() ; it != faces.end(); ++it){
+	MVertex *v1 = it->first.getVertex(0);
+	MVertex *v2 = it->first.getVertex(1);
+	MVertex *v3 = it->first.getVertex(2);
+	fprintf(f,"ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d};\n",v1->x(),v1->y(),v1->z(),v2->x(),v2->y(),v2->z(),
+		v3->x(),v3->y(),v3->z(),it->second->tag(),it->second->tag(),it->second->tag());
+      }
+      fprintf(f,"};\n");
+      fclose(f);
+    }
+  }
+};
+
 template<class T>
 static void GetQualityMeasure(std::vector<T*> &ele,
                               double &gamma, double &gammaMin, double &gammaMax,
                               double &minSICN, double &minSICNMin, double &minSICNMax,
-                              double &rho, double &rhoMin, double &rhoMax,
+                              double &minSIGE, double &minSIGEMin, double &minSIGEMax,
                               double quality[3][100])
 {
   for(unsigned int i = 0; i < ele.size(); i++){
@@ -65,14 +135,14 @@ static void GetQualityMeasure(std::vector<T*> &ele,
     minSICN += s;
     minSICNMin = std::min(minSICNMin, s);
     minSICNMax = std::max(minSICNMax, s);
-    double r = ele[i]->rhoShapeMeasure();
-    rho += r;
-    rhoMin = std::min(rhoMin, r);
-    rhoMax = std::max(rhoMax, r);
+    double e = ele[i]->minSIGEShapeMeasure();
+    minSIGE += e;
+    minSIGEMin = std::min(minSIGEMin, e);
+    minSIGEMax = std::max(minSIGEMax, e);
     for(int j = 0; j < 100; j++){
       if(s > (2*j-100) / 100. && s <= (2*j-98) / 100.) quality[0][j]++;
       if(g > j / 100. && g <= (j + 1) / 100.) quality[1][j]++;
-      if(r > j / 100. && r <= (j + 1) / 100.) quality[2][j]++;
+      if(e > (2*j-100) / 100. && e <= (2*j-98) / 100.) quality[2][j]++;
     }
   }
 }
@@ -124,35 +194,41 @@ void GetStatistics(double stat[50], double quality[3][100])
       for(int j = 0; j < 100; j++)
         quality[i][j] = 0.;
     double minSICN = 0., minSICNMin = 1., minSICNMax = -1.;
+    double minSIGE = 0., minSIGEMin = 1., minSIGEMax = -1.;
     double gamma = 0., gammaMin = 1., gammaMax = 0.;
-    double rho = 0., rhoMin = 1., rhoMax = 0.;
 
     double N = stat[9] + stat[10] + stat[11] + stat[12] + stat[13];
     if(N){ // if we have 3D elements
       for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
         GetQualityMeasure((*it)->tetrahedra, gamma, gammaMin, gammaMax,
-                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
+                          minSICN, minSICNMin, minSICNMax,
+                          minSIGE, minSIGEMin, minSIGEMax, quality);
         GetQualityMeasure((*it)->hexahedra, gamma, gammaMin, gammaMax,
-                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
+                          minSICN, minSICNMin, minSICNMax,
+                          minSIGE, minSIGEMin, minSIGEMax, quality);
         GetQualityMeasure((*it)->prisms, gamma, gammaMin, gammaMax,
-                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
+                          minSICN, minSICNMin, minSICNMax,
+                          minSIGE, minSIGEMin, minSIGEMax, quality);
         GetQualityMeasure((*it)->pyramids, gamma, gammaMin, gammaMax,
-                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
+                          minSICN, minSICNMin, minSICNMax,
+                          minSIGE, minSIGEMin, minSIGEMax, quality);
       }
     }
     else{ // 2D elements
       N = stat[7] + stat[8];
       for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
         GetQualityMeasure((*it)->quadrangles, gamma, gammaMin, gammaMax,
-                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
+                          minSICN, minSICNMin, minSICNMax,
+                          minSIGE, minSIGEMin, minSIGEMax, quality);
         GetQualityMeasure((*it)->triangles, gamma, gammaMin, gammaMax,
-                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
+                          minSICN, minSICNMin, minSICNMax,
+                          minSIGE, minSIGEMin, minSIGEMax, quality);
       }
     }
     if(N){
       stat[18] = minSICN / N; stat[19] = minSICNMin; stat[20] = minSICNMax;
       stat[21] = gamma / N;   stat[22] = gammaMin;   stat[23] = gammaMax;
-      stat[25] = rho / N;     stat[25] = rhoMin;     stat[26] = rhoMax;
+      stat[24] = minSIGE / N; stat[25] = minSIGEMin; stat[26] = minSIGEMax;
     }
   }
 
@@ -212,6 +288,9 @@ static bool CancelDelaunayHybrid(GModel *m)
 
 static void Mesh0D(GModel *m)
 {
+
+  m->getFields()->initialize();
+
   for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it){
     GVertex *gv = *it;
     if(gv->mesh_vertices.empty())
@@ -233,26 +312,43 @@ static void Mesh0D(GModel *m)
 
 static void Mesh1D(GModel *m)
 {
+
+  m->getFields()->initialize();
+
   if(TooManyElements(m, 1)) return;
   Msg::StatusBar(true, "Meshing 1D...");
   double t1 = Cpu();
 
-  for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it)
+
+  std::vector<GEdge*> temp;
+  for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it){
     (*it)->meshStatistics.status = GEdge::PENDING;
+    temp.push_back(*it);
+  }
 
   Msg::ResetProgressMeter();
 
   int nIter = 0, nTot = m->getNumEdges();
   while(1){
-    //    meshGEdge mesher;
     int nPending = 0;
-    for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it){
-      if ((*it)->meshStatistics.status == GEdge::PENDING){
-        (*it)->mesh(true);
-        nPending++;
+    const size_t sss = temp.size();
+#if defined(_OPENMP)
+#pragma omp parallel for schedule (dynamic)
+#endif
+    for(size_t K = 0 ; K < sss ; K++){
+      GEdge *ed = temp[K];
+      if (ed->meshStatistics.status == GEdge::PENDING){
+	ed->mesh(true);
+#if defined(_OPENMP)
+#pragma omp critical
+#endif
+	{
+	  nPending++;
+	}
       }
       if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 1D...");
     }
+
     if(!nPending) break;
     if(nIter++ > 10) break;
   }
@@ -328,9 +424,11 @@ static void PrintMesh2dStatistics(GModel *m)
 
 static void Mesh2D(GModel *m)
 {
+  m->getFields()->initialize();
+
   if(TooManyElements(m, 2)) return;
   Msg::StatusBar(true, "Meshing 2D...");
-  double t1 = GetTimeInSeconds();
+  double t1 = Cpu();
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
     (*it)->meshStatistics.status = GFace::PENDING;
@@ -430,18 +528,21 @@ static void Mesh2D(GModel *m)
 
   // collapseSmallEdges(*m);
 
-  double t2 = GetTimeInSeconds();
+  double t2 = Cpu();
   CTX::instance()->meshTimer[1] = t2 - t1;
   Msg::StatusBar(true, "Done meshing 2D (%g s)", CTX::instance()->meshTimer[1]);
 
   PrintMesh2dStatistics(m);
 }
 
-static void FindConnectedRegions(std::vector<GRegion*> &delaunay,
+static void FindConnectedRegions(const std::vector<GRegion*> &del,
                                  std::vector<std::vector<GRegion*> > &connected)
 {
+  std::vector<GRegion*> delaunay = del;
+  // test: connected.resize(1); connected[0] = delaunay; return;
+
   const unsigned int nbVolumes = delaunay.size();
-  if (!nbVolumes)return;
+  if (!nbVolumes) return;
   while (delaunay.size()){
     std::set<GRegion*> oneDomain;
     std::stack<GRegion*> _stack;
@@ -452,24 +553,24 @@ static void FindConnectedRegions(std::vector<GRegion*> &delaunay,
       _stack.pop();
       oneDomain.insert(r);
       std::list<GFace*> faces = r->faces();
-      for (std::list<GFace*> :: iterator it = faces.begin(); it != faces.end() ; ++it){
+      for (std::list<GFace*>::iterator it = faces.begin(); it != faces.end() ; ++it){
         GFace *gf = *it;
-        GRegion *other = gf->getRegion(0) == r ? gf->getRegion(1) : gf->getRegion(0);
-        if (other != 0 && oneDomain.find(other) == oneDomain.end())
-          _stack.push (other);
+        GRegion *other = (gf->getRegion(0) == r) ? gf->getRegion(1) : gf->getRegion(0);
+        if(other != 0 && oneDomain.find(other) == oneDomain.end())
+          _stack.push(other);
       }
     }
-    std::vector<GRegion*> temp1,temp2;
-    for (unsigned int i=0;i<delaunay.size();i++){
+    std::vector<GRegion*> temp1, temp2;
+    for (unsigned int i = 0; i < delaunay.size(); i++){
       r = delaunay[i];
-      if (oneDomain.find(r) == oneDomain.end())temp1.push_back(r);
+      if (oneDomain.find(r) == oneDomain.end()) temp1.push_back(r);
       else temp2.push_back(r);
     }
     connected.push_back(temp2);
-    delaunay=temp1;
+    delaunay = temp1;
   }
   Msg::Info("Delaunay Meshing %d volumes with %d connected components",
-            nbVolumes,connected.size());
+            nbVolumes, connected.size());
 }
 
 template <class ITERATOR>
@@ -718,10 +819,17 @@ void TestConformity(GModel *gm)
 
 static void Mesh3D(GModel *m)
 {
+  m->getFields()->initialize();
+
   if(TooManyElements(m, 3)) return;
   Msg::StatusBar(true, "Meshing 3D...");
   double t1 = Cpu();
 
+  Msg::ResetProgressMeter();
+
+  if(m->getNumRegions())
+    Msg::ProgressMeter(0, 100, false, "Meshing 3D...");
+
   // mesh the extruded volumes first
   std::for_each(m->firstRegion(), m->lastRegion(), meshGRegionExtruded());
 
@@ -835,9 +943,17 @@ static void Mesh3D(GModel *m)
   // ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
+  //  std::for_each(m->firstRegion(), m->lastRegion(), optimizeMeshGRegionNetgen());
+  if(Msg::GetVerbosity() > 98)
+    std::for_each(m->firstRegion(), m->lastRegion(),
+                  TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES());
+
   CTX::instance()->mesh.changed = ENT_ALL;
   double t2 = Cpu();
   CTX::instance()->meshTimer[2] = t2 - t1;
+
+  if(m->getNumRegions())
+    Msg::ProgressMeter(100, 100, false, "Meshing 3D...");
   Msg::StatusBar(true, "Done meshing 3D (%g s)", CTX::instance()->meshTimer[2]);
 }
 
@@ -850,6 +966,9 @@ void OptimizeMeshNetgen(GModel *m)
   // Ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
+  if (Msg::GetVerbosity() > 98)
+    std::for_each(m->firstRegion(), m->lastRegion(), TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES ());
+
   double t2 = Cpu();
   Msg::StatusBar(true, "Done optimizing 3D mesh with Netgen (%g s)", t2 - t1);
 }
@@ -863,6 +982,9 @@ void OptimizeMesh(GModel *m)
   // Ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
+  if (Msg::GetVerbosity() > 98)
+    std::for_each(m->firstRegion(), m->lastRegion(), TEST_IF_MESH_IS_COMPATIBLE_WITH_EMBEDDED_ENTITIES ());
+
   CTX::instance()->mesh.changed = ENT_ALL;
   double t2 = Cpu();
   Msg::StatusBar(true, "Done optimizing 3D mesh (%g s)", t2 - t1);
@@ -903,7 +1025,7 @@ void RecombineMesh(GModel *m)
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
     GFace *gf = *it;
-    recombineIntoQuads(gf);
+    recombineIntoQuads(gf,true,true,.01);
   }
 
   CTX::instance()->mesh.changed = ENT_ALL;
