@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -145,9 +145,6 @@ static double F_Lc_aniso(GEdge *ge, double t)
   bool blf = false;
 #endif
 
-  //printf("coucou\n");
-
-
   GPoint p = ge->point(t);
   SMetric3 lc_here;
 
@@ -447,79 +444,70 @@ static void filterPoints(GEdge*ge, int nMinimumPoints)
   }
 }
 
-static void createPoints ( GVertex *gv, GEdge *ge, BoundaryLayerField *blf ,
-			   std::vector<MVertex*>& v, const SVector3 &dir){
-  double L = blf->hwall_n;
-
+static void createPoints(GVertex *gv, GEdge *ge, BoundaryLayerField *blf,
+                         std::vector<MVertex*>& v, const SVector3 &dir)
+{
+#if defined(HAVE_ANN)
+  const double hwall = blf->hwall(gv->tag());
+  double L = hwall;
   double LEdge = distance (ge->getBeginVertex()->mesh_vertices[0],
 			   ge->getEndVertex()->mesh_vertices[0]);
-
   while (1){
     if (L > blf->thickness || L > LEdge * .4) break;
     SPoint3 p (gv->x() + dir.x() * L, gv->y() + dir.y() * L, 0.0);
-    v.push_back(new MEdgeVertex (p.x(), p.y(), p.z(), ge,  ge->parFromPoint(p), blf->hfar));
+    v.push_back(new MEdgeVertex(p.x(), p.y(), p.z(), ge,  ge->parFromPoint(p), blf->hfar));
     int ith = v.size() ;
-    L+= blf->hwall_n * pow (blf->ratio, ith);
-    //    printf("parameter %g length %g\n",ge->parFromPoint(p),L);
+    L += hwall * pow (blf->ratio, ith);
   }
+#endif
 }
 
-void addBoundaryLayerPoints (GEdge *ge,
-			     double &t_begin, // may change the left  parameter of the interval
-			     double &t_end,   // may change the right parameter of the interval
-			     std::vector<MVertex*> &_addBegin, // additional points @ left
-			     std::vector<MVertex*> &_addEnd)   // additional points @ right
+static void addBoundaryLayerPoints(GEdge *ge, double &t_begin, double &t_end,
+                                   std::vector<MVertex*> &_addBegin,
+                                   std::vector<MVertex*> &_addEnd)
 {
-  BoundaryLayerField *blf = 0;
 #if defined(HAVE_ANN)
+  // t_begin/t_end may change the left/right parameter of the interval
+  // _addBegin/_addEnd : additional points @ left/right
+  BoundaryLayerField *blf = 0;
   FieldManager *fields = ge->model()->getFields();
   Field *bl_field = fields->get(fields->getBoundaryLayerField());
   blf = dynamic_cast<BoundaryLayerField*> (bl_field);
-  if (blf) blf->setupFor1d(ge->tag());
-#endif
-  if (!blf) return;
-  if (blf->isEdgeBL(ge->tag()))return;
-  SVector3 dir ( ge->getEndVertex()->x() - ge->getBeginVertex()->x(),
-		 ge->getEndVertex()->y() - ge->getBeginVertex()->y(),
-		 ge->getEndVertex()->z() - ge->getBeginVertex()->z());
+  if(blf) blf->setupFor1d(ge->tag());
+  if(!blf) return;
+  if(blf->isEdgeBL(ge->tag())) return;
+  SVector3 dir(ge->getEndVertex()->x() - ge->getBeginVertex()->x(),
+               ge->getEndVertex()->y() - ge->getBeginVertex()->y(),
+               ge->getEndVertex()->z() - ge->getBeginVertex()->z());
   dir.normalize();
   GVertex *gvb = ge->getBeginVertex();
   GVertex *gve = ge->getEndVertex();
-  if (blf->isEndNode(gvb->tag())){
-    if (ge->geomType() != GEntity::Line){
-      Msg::Error ("Boundary layer end point %d should lie on a straight line", gvb->tag());
+  if(blf->isEndNode(gvb->tag())){
+    if(ge->geomType() != GEntity::Line){
+      Msg::Error("Boundary layer end point %d should lie on a straight line",
+                 gvb->tag());
       return;
     }
-    createPoints (gvb, ge, blf, _addBegin, dir);
-    if (!_addBegin.empty())_addBegin[_addBegin.size()-1]->getParameter(0,t_begin);
+    createPoints(gvb, ge, blf, _addBegin, dir);
+    if(!_addBegin.empty()) _addBegin[_addBegin.size()-1]->getParameter(0, t_begin);
   }
   if (blf->isEndNode(gve->tag())){
     if (ge->geomType() != GEntity::Line){
-      Msg::Error ("Boundary layer end point %d should lie on a straight line", gve->tag());
+      Msg::Error("Boundary layer end point %d should lie on a straight line",
+                 gve->tag());
       return;
     }
-    createPoints (gve, ge, blf, _addEnd, dir * -1.0);
-    if (!_addEnd.empty())_addEnd[_addEnd.size()-1]->getParameter(0,t_end);
+    createPoints(gve, ge, blf, _addEnd, dir * -1.0);
+    if(!_addEnd.empty()) _addEnd[_addEnd.size()-1]->getParameter(0, t_end);
   }
-
-  //  printf("Edge %d § %d %d points added (%g %g)-\n", ge->tag(), _addBegin.size(),_addEnd.size(),t_begin,t_end);
+#endif
 }
 
 void meshGEdge::operator() (GEdge *ge)
 {
-  /*
-  BoundaryLayerField *blf = 0;
-#if defined(HAVE_ANN)
-  FieldManager *fields = ge->model()->getFields();
-  Field *bl_field = fields->get(fields->getBoundaryLayerField());
-  blf = dynamic_cast<BoundaryLayerField*> (bl_field);
-  if (blf) blf->setupFor1d(ge->tag());
-#endif
-  */
-
   ge->model()->setCurrentMeshEntity(ge);
 
-  //  if(ge->geomType() == GEntity::DiscreteCurve) return;
+  // if(ge->geomType() == GEntity::DiscreteCurve) return;
   if(ge->geomType() == GEntity::BoundaryLayerCurve) return;
   if(ge->meshAttributes.method == MESH_NONE) return;
   if(CTX::instance()->mesh.meshOnlyVisible && !ge->getVisibility()) return;
@@ -533,7 +521,7 @@ void meshGEdge::operator() (GEdge *ge)
 
   if(MeshExtrudedCurve(ge)) return;
 
-  if (ge->meshMaster() != ge){
+  if(ge->meshMaster() != ge){
     GEdge *gef = dynamic_cast<GEdge*> (ge->meshMaster());
     if (gef->meshStatistics.status == GEdge::PENDING) return;
     Msg::Info("Meshing curve %d (%s) as a copy of %d", ge->tag(),
@@ -543,7 +531,27 @@ void meshGEdge::operator() (GEdge *ge)
     return;
   }
 
-  Msg::Info("Meshing curve %d (%s)", ge->tag(), ge->getTypeString().c_str());
+  if(ge->model()->getNumEdges() > 100000){
+    if (ge->tag() % 100000 == 1){
+      Msg::Info("Meshing curve %d/%d (%s)", ge->tag(), ge->model()->getNumEdges(),
+                ge->getTypeString().c_str());
+    }
+  }
+  else if(ge->model()->getNumEdges() > 10000){
+    if (ge->tag() % 10000 == 1){
+      Msg::Info("Meshing curve %d/%d (%s)", ge->tag(), ge->model()->getNumEdges(),
+                ge->getTypeString().c_str());
+    }
+  }
+  else if(ge->model()->getNumEdges() > 1000){
+    if (ge->tag() % 1000 == 1){
+      Msg::Info("Meshing curve %d/%d (%s)", ge->tag(), ge->model()->getNumEdges(),
+                ge->getTypeString().c_str());
+    }
+  }
+  else{
+    Msg::Info("Meshing curve %d (%s)", ge->tag(), ge->getTypeString().c_str());
+  }
 
   // compute bounds
   Range<double> bounds = ge->parBounds(0);
@@ -552,18 +560,19 @@ void meshGEdge::operator() (GEdge *ge)
 
   // if a BL is ending at one of the ends, then create specific points
   std::vector<MVertex*> _addBegin, _addEnd;
-  addBoundaryLayerPoints (ge, t_begin, t_end, _addBegin, _addEnd);
+  addBoundaryLayerPoints(ge, t_begin, t_end, _addBegin, _addEnd);
 
   // first compute the length of the curve by integrating one
   double length;
   std::vector<IntPoint> Points;
-  if(ge->geomType() == GEntity::Line &&
-      ge->getBeginVertex() == ge->getEndVertex() &&
-      //do not consider closed lines as degenerated
-      (ge->position(0.5) - ge->getBeginVertex()->xyz()).norm() < CTX::instance()->geom.tolerance)
-    length = 0.; // special case t avoid infinite loop in integration
+  if(ge->geomType() == GEntity::Line && ge->getBeginVertex() &&
+     ge->getBeginVertex() == ge->getEndVertex() &&
+     // do not consider closed lines as degenerated
+     (ge->position(0.5) - ge->getBeginVertex()->xyz()).norm() <
+     CTX::instance()->geom.tolerance)
+    length = 0.; // special case to avoid infinite loop in integration
   else
-    length = Integration(ge, t_begin, t_end, F_One, Points, 1.e-8 * CTX::instance()->lc);
+    length = Integration(ge, t_begin, t_end, F_One, Points, 1.e-12 * CTX::instance()->lc);
   ge->setLength(length);
   Points.clear();
 
@@ -576,7 +585,7 @@ void meshGEdge::operator() (GEdge *ge)
   int N;
   int filterMinimumN = 1;
   if(length == 0. && CTX::instance()->mesh.toleranceEdgeLength == 0.){
-    Msg::Warning("Curve %d has a zero length", ge->tag());
+    Msg::Debug("Curve %d has a zero length", ge->tag());
     a = 0.;
     N = 1;
   }
@@ -607,7 +616,8 @@ void meshGEdge::operator() (GEdge *ge)
       SVector3 der = ge->firstDer(pt.t);
       pt.xp = der.norm();
     }
-    a = smoothPrimitive(ge, sqrt(CTX::instance()->mesh.smoothRatio), Points);
+    if (CTX::instance()->mesh.algo2d != ALGO_2D_BAMG)
+      a = smoothPrimitive(ge, sqrt(CTX::instance()->mesh.smoothRatio), Points);
     filterMinimumN = ge->minimumMeshSegments() + 1;
     N = std::max(filterMinimumN, (int)(a + 1.99));
   }
@@ -634,7 +644,6 @@ void meshGEdge::operator() (GEdge *ge)
     }
   }
 
-
   //printFandPrimitive(ge->tag(),Points);
 
   // if the curve is periodic and if the begin vertex is identical to
@@ -643,8 +652,12 @@ void meshGEdge::operator() (GEdge *ge)
   // curve. So, the mesh vertex and its associated geom vertex are not
   // necessary at the same location
   GPoint beg_p, end_p;
-  if(ge->getBeginVertex() == ge->getEndVertex() &&
-     ge->getBeginVertex()->edges().size() == 1){
+  if(!ge->getBeginVertex() && !ge->getEndVertex()){
+    Msg::Warning("Skipping curve with no begin nor end vertex");
+    return;
+  }
+  else if(ge->getBeginVertex() == ge->getEndVertex() &&
+          ge->getBeginVertex()->edges().size() == 1){
     end_p = beg_p = ge->point(t_begin);
     Msg::Debug("Meshing periodic closed curve");
   }
@@ -697,10 +710,9 @@ void meshGEdge::operator() (GEdge *ge)
     mesh_vertices = vv;
   }
 
-  //  printf("%ld ----> ", ge->mesh_vertices.size());
-  if (_addBegin.empty() && _addEnd.empty())
-    filterPoints (ge, filterMinimumN - 2);
-  //  printf("%ld \n", ge->mesh_vertices.size());
+  if (CTX::instance()->mesh.algo2d != ALGO_2D_BAMG)
+    if (_addBegin.empty() && _addEnd.empty())
+      filterPoints(ge, filterMinimumN - 2);
 
   for(unsigned int i = 0; i < mesh_vertices.size() + 1; i++){
     MVertex *v0 = (i == 0) ?
