@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -26,8 +26,6 @@
 #include "MTriangle.h"
 #include "MQuadrangle.h"
 #include "MElementCut.h"
-#include "CenterlineField.h"
-#include "meshGFaceElliptic.h"
 #include "Context.h"
 #include "GPoint.h"
 #include "GmshMessage.h"
@@ -556,10 +554,12 @@ static bool recoverEdge(BDS_Mesh *m, GEdge *ge,
         BDS_Edge *e = m->recover_edge(pstart->iD, pend->iD, _fatallyFailed, e2r, notRecovered);
         if(e) e->g = g;
         else {
-          if (_fatallyFailed)
-            Msg::Error("Unable to recover an edge %g %g && %g %g (%d/%d)",
-                       vstart->x(), vstart->y(), vend->x(), vend->y(), i,
-                       ge->mesh_vertices.size());
+          if (_fatallyFailed){
+            Msg::Error("Unable to recover the edge %d (%d/%d) on GEdge %d (on GFace %d)",
+                       ge->lines[i]->getNum(),i+1,ge->lines.size(),ge->tag(),ge->faces().back()->tag());
+	    outputScalarField(m->triangles, "wrongmesh.pos", 0);
+	    outputScalarField(m->triangles,"wrongparam.pos",1);
+	  }
           return !_fatallyFailed;
         }
       }
@@ -633,7 +633,8 @@ void BDS2GMSH(BDS_Mesh *m, GFace *gf,
   }
 }
 
-static void addOrRemove(MVertex *v1, MVertex *v2, std::set<MEdge,Less_Edge> & bedges, std::set<MEdge,Less_Edge> &removed)
+static void addOrRemove(MVertex *v1, MVertex *v2, std::set<MEdge,Less_Edge> & bedges,
+                        std::set<MEdge,Less_Edge> &removed)
 {
   MEdge e(v1,v2);
   if (removed.find(e) != removed.end())return;
@@ -645,7 +646,10 @@ static void addOrRemove(MVertex *v1, MVertex *v2, std::set<MEdge,Less_Edge> & be
   }
 }
 
-static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
+static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf,
+                                                                std::vector<MQuadrangle*> &blQuads,
+                                                                std::vector<MTriangle*> &blTris,
+                                                                std::set<MVertex*> &verts)
 {
   if (!buildAdditionalPoints2D (gf))return;
   BoundaryLayerColumns* _columns = gf->getColumns();
@@ -653,15 +657,12 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
   std::set<MEdge,Less_Edge> bedges;
   std::set<MEdge,Less_Edge> removed;
 
-  std::vector<MQuadrangle*> blQuads;
-  std::vector<MTriangle*> blTris;
   std::list<GEdge*> edges = gf->edges();
   std::list<GEdge*> embedded_edges = gf->embeddedEdges();
   edges.insert(edges.begin(), embedded_edges.begin(),embedded_edges.end());
   std::list<GEdge*>::iterator ite = edges.begin();
   FILE *ff2 = Fopen ("tato.pos","w");
   if(ff2) fprintf(ff2,"View \" \"{\n");
-  std::set<MVertex*> verts;
 
   std::vector<MLine*> _lines;
 
@@ -701,6 +702,22 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
                     v22->x(),v22->y(),v22->z(),
                     v21->x(),v21->y(),v21->z(),l+1,l+1,l+1,l+1);
         }
+        if (c1._column.size() != c2._column.size()) {
+          MVertex *v11,*v12,*v;
+          v11 = c1._column[N-1];
+          v12 = c2._column[N-1];
+          v = c1._column.size() > c2._column.size() ?
+              c1._column[N] : c2._column[N];
+          MTriangle *qq = new MTriangle(v11,v12,v);
+          qq->setPartition (N+1);
+          myCol.push_back(qq);
+          blTris.push_back(qq);
+          if(ff2)
+            fprintf(ff2,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d};\n",
+                    v11->x(),v11->y(),v11->z(),
+                    v12->x(),v12->y(),v12->z(),
+                    v->x(),v->y(),v->z(),N+1,N+1,N+1);
+        }
         // int M = std::max(c1._column.size(),c2._column.size());
         for (unsigned int l=0;l<myCol.size();l++)_columns->_toFirst[myCol[l]] = myCol[0];
         _columns->_elemColumns[myCol[0]] = myCol;
@@ -807,17 +824,13 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
   deMeshGFace kil_;
   kil_(gf);
   meshGenerator(gf, 0, 0, true , false, &hop);
-
-  gf->quadrangles = blQuads;
-  gf->triangles.insert(gf->triangles.begin(),blTris.begin(),blTris.end());
-  gf->mesh_vertices.insert(gf->mesh_vertices.begin(),verts.begin(),verts.end());
 }
 
 static bool inside_domain(MElementOctree* octree,double x,double y)
 {
   MElement* element;
-  element = (MElement*)octree->find(x,y,0.0,2,true);
-  if(element!=NULL) return 1;
+  element = (MElement*)octree->find(x, y, 0.0, 2, true);
+  if(element != NULL) return 1;
   else return 0;
 }
 
@@ -993,6 +1006,8 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     if (isMeshed) return true;
   }
 
+  //  if (gf->degenerate(0))return 0;
+  
   // build a set with all points of the boundaries
   std::set<MVertex*, MVertexLessThanNum> all_vertices, boundary;
   std::list<GEdge*>::iterator ite = edges.begin();
@@ -1396,7 +1411,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   }
 
   // compute characteristic lengths at vertices
-  if (!onlyInitialMesh){
+  if (CTX::instance()->mesh.algo2d != ALGO_2D_BAMG && !onlyInitialMesh){
       Msg::Debug("Computing mesh size field at mesh vertices %d",
                  edgesToRecover.size());
       std::set<BDS_Point*, PointLessThan>::iterator it = m->points.begin();
@@ -1531,6 +1546,10 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   // fill the small gmsh structures
   BDS2GMSH(m, gf, recoverMap);
 
+  std::vector<MQuadrangle*> blQuads;
+  std::vector<MTriangle*> blTris;
+  std::set<MVertex*> verts;
+
   bool infty = false;
   if (gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
       gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS ||
@@ -1540,7 +1559,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     if (infty)
       buildBackGroundMesh (gf);
     // BOUNDARY LAYER
-    modifyInitialMeshForTakingIntoAccountBoundaryLayers(gf);
+    modifyInitialMeshForTakingIntoAccountBoundaryLayers(gf, blQuads, blTris, verts);
   }
 
   // the delaunay algo is based directly on internal gmsh structures
@@ -1581,9 +1600,18 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
 
   delete m;
 
+
+
+  gf->quadrangles.insert(gf->quadrangles.begin(),blQuads.begin(),blQuads.end());
+  gf->triangles.insert(gf->triangles.begin(),blTris.begin(),blTris.end());
+  gf->mesh_vertices.insert(gf->mesh_vertices.begin(),verts.begin(),verts.end());
+
+  
   if((CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine) &&
      !CTX::instance()->mesh.optimizeLloyd && !onlyInitialMesh && CTX::instance()->mesh.algoRecombine != 2)
     recombineIntoQuads(gf);
+  
+
 
   computeElementShapes(gf, gf->meshStatistics.worst_element_shape,
                        gf->meshStatistics.average_element_shape,
@@ -1793,14 +1821,18 @@ static bool buildConsecutiveListOfVertices(GFace *gf, GEdgeLoop &gel,
 
     std::vector<MVertex*> edgeLoop;
     if(found._sign == 1){
-      edgeLoop.push_back(found.ge->getBeginVertex()->mesh_vertices[0]);
-      for(unsigned int i = 0; i <found.ge->mesh_vertices.size(); i++)
-        edgeLoop.push_back(found.ge->mesh_vertices[i]);
+      if(found.ge->getBeginVertex()){
+        edgeLoop.push_back(found.ge->getBeginVertex()->mesh_vertices[0]);
+        for(unsigned int i = 0; i < found.ge->mesh_vertices.size(); i++)
+          edgeLoop.push_back(found.ge->mesh_vertices[i]);
+      }
     }
     else{
-      edgeLoop.push_back(found.ge->getEndVertex()->mesh_vertices[0]);
-      for(int i = found.ge->mesh_vertices.size() - 1; i >= 0; i--)
-        edgeLoop.push_back(found.ge->mesh_vertices[i]);
+      if(found.ge->getEndVertex()){
+        edgeLoop.push_back(found.ge->getEndVertex()->mesh_vertices[0]);
+        for(int i = found.ge->mesh_vertices.size() - 1; i >= 0; i--)
+          edgeLoop.push_back(found.ge->mesh_vertices[i]);
+      }
     }
 
     if(MYDEBUG)
@@ -1811,31 +1843,45 @@ static bool buildConsecutiveListOfVertices(GFace *gf, GEdgeLoop &gel,
     for(unsigned int i = 0; i < edgeLoop.size(); i++){
       MVertex *here = edgeLoop[i];
       GEntity *ge = here->onWhat();
-      double U, V;
-      SPoint2 param = coords[i];
-      U = param.x() / m->scalingU ;
-      V = param.y() / m->scalingV;
-      BDS_Point *pp = m->add_point(count + countTot, U, V, gf);
-      if(ge->dim() == 0){
-        pp->lcBGM() = BGM_MeshSize(ge, 0, 0, here->x(), here->y(), here->z());
-      }
-      else if(ge->dim() == 1){
-        double u;
-        here->getParameter(0, u);
-        pp->lcBGM() = BGM_MeshSize(ge, u, 0,here->x(), here->y(), here->z());
-      }
-      else
-        pp->lcBGM() = MAX_LC;
 
-      pp->lc() = pp->lcBGM();
-      m->add_geom (ge->tag(), ge->dim());
-      BDS_GeomEntity *g = m->get_geom(ge->tag(), ge->dim());
-      pp->g = g;
-      if(MYDEBUG)
-        printf("point %3d (%8.5f %8.5f : %8.5f %8.5f) (%2d,%2d)\n",
-               count, pp->u, pp->v, param.x(), param.y(), pp->g->classif_tag,
-               pp->g->classif_degree);
-      bbox += SPoint3(U, V, 0);
+      BDS_Point *pp = 0;
+      if(ge->dim() == 0){
+        // Point might already be part of other loop
+        for (std::map<BDS_Point*, MVertex*, PointLessThan>::iterator it = recoverMap.begin();
+             it != recoverMap.end(); ++it){
+          if (it->second == here){
+            pp = it->first;
+            break;
+          }
+        }
+      }
+      if(pp == 0){
+        double U, V;
+        SPoint2 param = coords[i];
+        U = param.x() / m->scalingU;
+        V = param.y() / m->scalingV;
+        pp = m->add_point(count + countTot, U, V, gf);
+        if (ge->dim() == 0){
+          pp->lcBGM() = BGM_MeshSize(ge, 0, 0, here->x(), here->y(), here->z());
+        }
+        else if (ge->dim() == 1){
+          double u;
+          here->getParameter(0, u);
+          pp->lcBGM() = BGM_MeshSize(ge, u, 0, here->x(), here->y(), here->z());
+        }
+        else
+          pp->lcBGM() = MAX_LC;
+
+        pp->lc() = pp->lcBGM();
+        m->add_geom(ge->tag(), ge->dim());
+        BDS_GeomEntity *g = m->get_geom(ge->tag(), ge->dim());
+        pp->g = g;
+        if (MYDEBUG)
+          printf("point %3d (%8.5f %8.5f : %8.5f %8.5f) (%2d,%2d)\n",
+                 count, pp->u, pp->v, param.x(), param.y(), pp->g->classif_tag,
+                 pp->g->classif_degree);
+        bbox += SPoint3(U, V, 0);
+      }
       edgeLoop_BDS.push_back(pp);
       recoverMapLocal[pp] = here;
       count++;
@@ -1851,32 +1897,6 @@ static bool buildConsecutiveListOfVertices(GFace *gf, GEdgeLoop &gel,
   return true;
 }
 
-static bool meshGeneratorElliptic(GFace *gf, bool debug = true)
-{
-#if defined(HAVE_ANN)
-  Centerline *center = 0;
-  FieldManager *fields = GModel::current()->getFields();
-  if (fields->getBackgroundField() > 0 ){
-    Field *myField = fields->get(fields->getBackgroundField());
-    center = dynamic_cast<Centerline*> (myField);
-  }
-
-  bool recombine =  (CTX::instance()->mesh.recombineAll) ;
-  int nbBoundaries = gf->edges().size();
-
-  if (center && recombine && nbBoundaries == 2) {
-    printf("--> regular periodic grid generator (elliptic smooth) \n");
-    //bool success  = createRegularTwoCircleGrid(center, gf);
-    bool success  = createRegularTwoCircleGridPeriodic(center, gf);
-    return success;
-  }
-  else return false;
-
-#else
-  return false;
-#endif
-}
-
 static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
 {
   std::map<BDS_Point*, MVertex*, PointLessThan> recoverMap;
@@ -2341,7 +2361,13 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
   if (infty)
     buildBackGroundMesh (gf, &equivalence, &parametricCoordinates);
   // BOUNDARY LAYER
-  modifyInitialMeshForTakingIntoAccountBoundaryLayers(gf);
+  std::vector<MQuadrangle*> blQuads;
+  std::vector<MTriangle*> blTris;
+  std::set<MVertex*> verts;
+  modifyInitialMeshForTakingIntoAccountBoundaryLayers(gf, blQuads, blTris, verts);
+  gf->quadrangles.insert(gf->quadrangles.begin(),blQuads.begin(),blQuads.end());
+  gf->triangles.insert(gf->triangles.begin(),blTris.begin(),blTris.end());
+  gf->mesh_vertices.insert(gf->mesh_vertices.begin(),verts.begin(),verts.end());
 
 
   if(algoDelaunay2D(gf)){
@@ -2389,7 +2415,7 @@ void deMeshGFace::operator() (GFace *gf)
 }
 
 // for debugging, change value from -1 to -100;
-int debugSurface = -1; //-1;
+int debugSurface = -1; //-100;
 
 void meshGFace::operator() (GFace *gf, bool print)
 {
@@ -2458,11 +2484,6 @@ void meshGFace::operator() (GFace *gf, bool print)
 
   Msg::Debug("Generating the mesh");
 
-  if(meshGeneratorElliptic(gf)){
-    gf->meshStatistics.status = GFace::DONE;
-    return;
-  }
-
   quadMeshRemoveHalfOfOneDMesh halfmesh (gf);
 
   if ((gf->getNativeType() != GEntity::AcisModel ||
@@ -2523,7 +2544,7 @@ bool checkMeshCompound(GFaceCompound *gf, std::list<GEdge*> &edges)
 
 void partitionAndRemesh(GFaceCompound *gf)
 {
-#if defined(HAVE_SOLVER) && (defined(HAVE_CHACO) || defined(HAVE_METIS))
+#if defined(HAVE_SOLVER) && defined(HAVE_ANN) && (defined(HAVE_CHACO) || defined(HAVE_METIS))
 
   // Partition the mesh and createTopology for new faces
   double tbegin = Cpu();
