@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -38,6 +38,7 @@
 #include "Levy3D.h"
 #include "discreteFace.h"
 #include "filterElements.h"
+#include "ExtrudeParams.h"
 
 #if defined(HAVE_ANN)
 #include "ANN/ANN.h"
@@ -639,9 +640,9 @@ void MeshDelaunayVolumeTetgen(std::vector<GRegion*> &regions)
 
   // now do insertion of points
   if(CTX::instance()->mesh.algo3d == ALGO_3D_FRONTAL_DEL)
-    bowyerWatsonFrontalLayers(gr, false);
+    Msg::Error ("Frontal Layers Algo deprecated");
   else if(CTX::instance()->mesh.algo3d == ALGO_3D_FRONTAL_HEX)
-    bowyerWatsonFrontalLayers(gr, true);
+    Msg::Error ("Frontal Layers Algo deprecated");
   else if(CTX::instance()->mesh.algo3d == ALGO_3D_MMG3D){
     refineMeshMMG(gr);
   }
@@ -652,6 +653,7 @@ void MeshDelaunayVolumeTetgen(std::vector<GRegion*> &regions)
       insertVerticesInRegion(gr,2000000000,true);
     }
   }
+
   // crete an initial mesh
   if (sqr.buildPyramids (gr->model())){
     RelocateVertices(regions, 3);
@@ -672,7 +674,7 @@ static void MeshDelaunayVolumeNewCode(std::vector<GRegion*> &regions)
 {
   GRegion *gr = regions[0];
   std::list<GFace*> faces = gr->faces();
-  std::set<GFace*> allFacesSet;
+  std::set<GFace*, GEntityLessThan> allFacesSet;
   for(unsigned int i = 0; i < regions.size(); i++){
     std::list<GFace*> f = regions[i]->faces();
     allFacesSet.insert(f.begin(), f.end());
@@ -744,13 +746,13 @@ static void MeshDelaunayVolumeNewCode(std::vector<GRegion*> &regions)
   }
   else{
     void edgeBasedRefinement(const int numThreads,
-                             const int nptsatonce,
-                             GRegion *gr);
+			     const int nptsatonce,
+			     GRegion *gr);
     // just to remove tets that are not to be meshed
     insertVerticesInRegion(gr, 0);
     for(unsigned int i = 0; i < regions.size(); i++){
-      Msg::Info("Refining volume %d",regions[i]->tag());
-      edgeBasedRefinement(1, 1, regions[i]);
+      Msg::Info("Refining volume %d with %d threads",regions[i]->tag(),Msg::GetMaxThreads());
+      edgeBasedRefinement(Msg::GetMaxThreads(), 1, regions[i]);
     }
     // RelocateVertices (regions,-1);
   }
