@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -107,7 +107,7 @@ bool tetgenmesh::reconstructmesh(void *p)
 
   double t_start = Cpu();
   std::vector<MVertex*> _vertices;
-
+  std::map<int,MVertex*> _extras;
   // Get the set of vertices from GRegion.
   {
     std::set<MVertex*, MVertexLessThanNum> all;
@@ -425,8 +425,11 @@ bool tetgenmesh::reconstructmesh(void *p)
     // Construct a map from points to subfaces.
     makepoint2submap(subfaces, idx2shlist, shperverlist);
 
+    //    printf("coucou1\n");
+
     // Process the set of PSC edges.
     // Remeber that all segments have default marker '-1'.
+    //    int COUNTER = 0;
     for (std::list<GEdge*>::iterator it = e_list.begin(); it != e_list.end();
 	 ++it) {
       GEdge *ge = *it;
@@ -480,7 +483,7 @@ bool tetgenmesh::reconstructmesh(void *p)
 	  // It is a dangling segment (not belong to any facets).
 	  // Check if segment [p[0],p[1]] already exists.
 	  // TODO: Change the brute-force search. Slow!
-	  point *ppt;
+	  /*	  point *ppt;
 	  subsegs->traversalinit();
 	  segloop.sh = shellfacetraverse(subsegs);
 	  while (segloop.sh != NULL) {
@@ -492,7 +495,7 @@ bool tetgenmesh::reconstructmesh(void *p)
 	      break;
 	    }
 	    segloop.sh = shellfacetraverse(subsegs);
-	  }
+	    }*/
 	  if (newseg.sh == NULL) {
 	    makeshellface(subsegs, &newseg);
 	    setshvertices(newseg, p[0], p[1], NULL);
@@ -522,9 +525,13 @@ bool tetgenmesh::reconstructmesh(void *p)
   // Boundary recovery.
 
   clock_t t;
+  //    printf("coucou2\n");
+  Msg::Info("Boundary Recovery...");
   recoverboundary(t);
+  //    printf("coucou3\n");
 
   carveholes();
+  //    printf("coucou4\n");
 
   if (subvertstack->objects > 0l) {
     suppresssteinerpoints();
@@ -645,7 +652,7 @@ bool tetgenmesh::reconstructmesh(void *p)
               }
               v->setIndex(pointmark(pointloop));
               _gr->mesh_vertices.push_back(v);
-              _vertices.push_back(v);
+	      _extras[pointmark(pointloop)] = v;
             }
             spivot(parentseg, parentsh);
             if (parentsh.sh != NULL) {
@@ -669,7 +676,7 @@ bool tetgenmesh::reconstructmesh(void *p)
                   }
                   v->setIndex(pointmark(pointloop));
                   _gr->mesh_vertices.push_back(v);
-                  _vertices.push_back(v);
+		  _extras[pointmark(pointloop)] = v;
                 }
               }
               // Record all the GFaces' tag at this segment.
@@ -684,9 +691,8 @@ bool tetgenmesh::reconstructmesh(void *p)
               // Create an interior mesh vertex.
               MVertex *v = new MVertex(pointloop[0], pointloop[1], pointloop[2], _gr);
               v->setIndex(pointmark(pointloop));
-              _gr->mesh_vertices.push_back(v);
-              _vertices.push_back(v);
-            }
+	      _extras[pointmark(pointloop)] = v;
+	      _gr->mesh_vertices.push_back(v);            }
           }
           else if (pointtype(pointloop) == FREEFACETVERTEX) {
             sdecode(point2sh(pointloop), parentsh);
@@ -712,29 +718,34 @@ bool tetgenmesh::reconstructmesh(void *p)
               }
               v->setIndex(pointmark(pointloop));
               _gr->mesh_vertices.push_back(v);
-              _vertices.push_back(v);
+	      _extras[pointmark(pointloop)] = v;
             }
             else {
               // Create a mesh vertex.
               MVertex *v = new MVertex(pointloop[0], pointloop[1], pointloop[2], _gr);
               v->setIndex(pointmark(pointloop));
               _gr->mesh_vertices.push_back(v);
-              _vertices.push_back(v);
+	      _extras[pointmark(pointloop)] = v;
             }
           }
           else {
             MVertex *v = new MVertex(pointloop[0], pointloop[1], pointloop[2], _gr);
             v->setIndex(pointmark(pointloop));
             _gr->mesh_vertices.push_back(v);
-            _vertices.push_back(v);
+	    _extras[pointmark(pointloop)] = v;
           }
         }
         pointloop = pointtraverse();
       }
-      assert((int)_vertices.size() == points->items);
+      //      assert((int)_vertices.size() == points->items);
     }
 
+    if (!_extras.empty())Msg::Info("We add %d steiner points...",_extras.size());
+
+
     if (l_edges.size() > 0) {
+
+
       // There are Steiner points on segments!
       face segloop;
       // Re-create the segment mesh in the corresponding GEdges.
@@ -750,6 +761,7 @@ bool tetgenmesh::reconstructmesh(void *p)
           }
         }
         assert(ge != NULL);
+	// Msg::Info("Steiner points exist on GEdge %d", ge->tag());
         // Delete the old triangles.
         for(unsigned int i = 0; i < ge->lines.size(); i++)
           delete ge->lines[i];
@@ -763,8 +775,13 @@ bool tetgenmesh::reconstructmesh(void *p)
           if (shellmark(segloop) == etag) {
             p[0] = sorg(segloop);
             p[1] = sdest(segloop);
-            MVertex *v1 = _vertices[pointmark(p[0])];
-            MVertex *v2 = _vertices[pointmark(p[1])];
+            MVertex *v1 = pointmark(p[0]) >= (int)_vertices.size() ?
+              _extras[pointmark(p[0])] : _vertices[pointmark(p[0])];
+            MVertex *v2 = pointmark(p[1]) >= (int)_vertices.size() ?
+              _extras[pointmark(p[1])] : _vertices[pointmark(p[1])];
+	    // MVertex *v2 = _vertices[pointmark(p[1])];
+	    // printf("%d %d %d\n",pointmark(p[0]),pointmark(p[1]),_vertices.size());
+	    // printf("%g %g %g %g\n",v1->x(),v1->y(),v2->x(),v2->y());
             MLine *t = new MLine(v1, v2);
             ge->lines.push_back(t);
           }
@@ -807,9 +824,21 @@ bool tetgenmesh::reconstructmesh(void *p)
             p[0] = sorg(subloop);
             p[1] = sdest(subloop);
             p[2] = sapex(subloop);
-            MVertex *v1 = _vertices[pointmark(p[0])];
-            MVertex *v2 = _vertices[pointmark(p[1])];
-            MVertex *v3 = _vertices[pointmark(p[2])];
+            MVertex *v1 = pointmark(p[0]) >= (int)_vertices.size() ?
+              _extras[pointmark(p[0])] : _vertices[pointmark(p[0])];
+            MVertex *v2 = pointmark(p[1]) >= (int)_vertices.size() ?
+              _extras[pointmark(p[1])] : _vertices[pointmark(p[1])];
+            MVertex *v3 = pointmark(p[2]) >= (int)_vertices.size() ?
+              _extras[pointmark(p[2])] : _vertices[pointmark(p[2])];
+	    // if (pointmark(p[0]) >= _vertices.size())
+            //   printf("F %d %d\n",v1->getIndex(),pointmark(p[0]));
+	    // if (pointmark(p[1]) >= _vertices.size())
+            //   printf("F %d %d\n",v2->getIndex(),pointmark(p[1]));
+	    // if (pointmark(p[2]) >= _vertices.size())
+            //   printf("F %d %d\n",v3->getIndex(),pointmark(p[2]));
+	    // MVertex *v1 = _vertices[pointmark(p[0])];
+	    // MVertex *v2 = _vertices[pointmark(p[1])];
+	    // MVertex *v3 = _vertices[pointmark(p[2])];
             MTriangle *t = new MTriangle(v1, v2, v3);
             gf->triangles.push_back(t);
           }
@@ -830,10 +859,26 @@ bool tetgenmesh::reconstructmesh(void *p)
       p[2] = apex(tetloop);
       p[3] = oppo(tetloop);
 
-      MVertex *v1 = _vertices[pointmark(p[0])];
-      MVertex *v2 = _vertices[pointmark(p[1])];
-      MVertex *v3 = _vertices[pointmark(p[2])];
-      MVertex *v4 = _vertices[pointmark(p[3])];
+      MVertex *v1 = pointmark(p[0]) >= (int)_vertices.size() ?
+        _extras[pointmark(p[0])] : _vertices[pointmark(p[0])];
+      MVertex *v2 = pointmark(p[1]) >= (int)_vertices.size() ?
+        _extras[pointmark(p[1])] : _vertices[pointmark(p[1])];
+      MVertex *v3 = pointmark(p[2]) >= (int)_vertices.size() ?
+        _extras[pointmark(p[2])] : _vertices[pointmark(p[2])];
+      MVertex *v4 = pointmark(p[3]) >= (int)_vertices.size() ?
+        _extras[pointmark(p[3])] : _vertices[pointmark(p[3])];
+      // if (pointmark(p[0]) >= _vertices.size())
+      //   printf("%d %d\n",v1->getIndex(),pointmark(p[0]));
+      // if (pointmark(p[1]) >= _vertices.size())
+      //   printf("%d %d\n",v2->getIndex(),pointmark(p[1]));
+      // if (pointmark(p[2]) >= _vertices.size())
+      //   printf("%d %d\n",v3->getIndex(),pointmark(p[2]));
+      // if (pointmark(p[3]) >= _vertices.size())
+      ///   printf("%d %d\n",v4->getIndex(),pointmark(p[3]));
+      // MVertex *v1 = _vertices[pointmark(p[0])];
+      // MVertex *v2 = _vertices[pointmark(p[1])];
+      // MVertex *v3 = _vertices[pointmark(p[2])];
+      // MVertex *v4 = _vertices[pointmark(p[3])];
       MTetrahedron *t = new  MTetrahedron(v1, v2, v3, v4);
       _gr->tetrahedra.push_back(t);
       tetloop.tet = tetrahedrontraverse();
