@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -658,16 +658,20 @@ void invert_singular_matrix3x3(double MM[3][3], double II[3][3])
 bool newton_fd(bool (*func)(fullVector<double> &, fullVector<double> &, void *),
                fullVector<double> &x, void *data, double relax, double tolx)
 {
-  const int MAXIT = 10;
+  const int MAXIT = 100;
   const double EPS = 1.e-4;
   const int N = x.size();
 
   fullMatrix<double> J(N, N);
   fullVector<double> f(N), feps(N), dx(N);
 
-  for (int iter = 0; iter < MAXIT; iter++){
-    if(!func(x, f, data)) return false;
-
+  
+  for (int iter = 0; iter < MAXIT; iter++){    
+    if (x.norm() > 1.e6)return false;
+    if(!func(x, f, data)) {
+      return false;
+    }
+      
     bool isZero = false;
     for (int k=0; k<N; k++) {
       if (f(k) == 0. ) isZero = true;
@@ -680,25 +684,28 @@ bool newton_fd(bool (*func)(fullVector<double> &, fullVector<double> &, void *),
       double h = EPS * fabs(x(j));
       if(h == 0.) h = EPS;
       x(j) += h;
-      if(!func(x, feps, data)) return false;
+      if(!func(x, feps, data)) {
+	return false;
+      }
       for (int i = 0; i < N; i++){
         J(i, j) = (feps(i) - f(i)) / h;
       }
       x(j) -= h;
     }
 
-    //    printf("done\n");
-
     if (N == 1)
       dx(0) = f(0) / J(0, 0);
     else
-      if (!J.luSolve(f, dx))
+      if (!J.luSolve(f, dx)){
         return false;
+      }
 
     for (int i = 0; i < N; i++)
       x(i) -= relax * dx(i);
 
-    if(dx.norm() < tolx) return true;
+    if(dx.norm() < tolx) {
+      return true;
+    }
   }
   return false;
 }
@@ -1505,3 +1512,50 @@ void transformPointsIntoOrthoBasis(const std::vector<SPoint3> &ptsProj,
     pointsUV[i][2] = dot(pp, normal);
   }
 }
+
+static bool catenary_fct(fullVector<double> &x, fullVector<double> &res, void *data)
+{
+  double *param = (double*)data;
+  double x0 = param[0], x1 = param[1], y0 = param[2], y1 = param[3], ys = param[4];
+  res(0) = (ys-1/x(0)) + 1/x(0)*cosh(x(0)*(x0-x(1))) - y0;
+  res(1) = (ys-1/x(0)) + 1/x(0)*cosh(x(0)*(x1-x(1))) - y1;
+  return true;
+}
+
+bool catenary(double x0, double x1, double y0, double y1, double ys, int N,
+              double *yp)
+{
+  // In the z=0 plane, catenary equation is y(x) = a + 1/b cosh(b(x-c))
+  //
+  // Three parameters a, b, c determined by imposing
+  // - left point: y0 = y(x0) = a + 1/b cosh(b(x0-c))
+  // - right point: y1 = y(x1) = a + 1/b cosh(b(x1-c))
+  // - lowest point (sag): ys = y(c) = a + 1/b , i.e. a = ys - 1/b
+  //
+  // Thus solve syst of 2 nl equations with 2 unknowns b and c:
+  //
+  // ys - 1/b + 1/b cosh(b(x0-c)) - y0 = 0
+  // ys - 1/b + 1/b cosh(b(x1-c)) - y1 = 0
+  double param[5] = {x0, x1, y0, y1, ys};
+  fullVector<double> x(2);
+  bool success = false;
+  if(x0 != x1){
+    x(0) = 1./(x1-x0);
+    x(1) = (x0+x1)/2.;
+    success = newton_fd(catenary_fct, x, param, 1., 1e-6*fabs(x1-x0));
+  }
+  if(success){
+    double a = ys-1/x(0);
+    for(int i = 0; i < N; i++){
+      double r = x0 + (i+1) * (x1-x0)/(N+1);
+      yp[i] = a+1/x(0)*cosh(x(0)*(r-x(1)));
+    }
+    return true;
+  }
+  else{
+    for(int i = 0; i < N; i++){
+      yp[i] = y0 + (i+1) * (y1-y0)/(N+1);
+    }
+    return false;
+  }
+}
